#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

from __future__ import division
import threading
from datetime import datetime

import os.path
import pygtk
import gobject
import gtk
import cgi

from rabbitvcs.ui import InterfaceView
from rabbitvcs.ui.action import SVNAction, GitAction, vcs_action_factory
from rabbitvcs.ui.dialog import MessageBox
from rabbitvcs.util.contextmenu import GtkContextMenu
from rabbitvcs.util.contextmenuitems import *
import rabbitvcs.ui.widget
import rabbitvcs.util.helper
import rabbitvcs.vcs
from rabbitvcs.util.decorators import gtk_unsafe

from rabbitvcs import gettext
_ = gettext.gettext

DATETIME_FORMAT = rabbitvcs.util.helper.LOCAL_DATETIME_FORMAT

REVISION_LABEL = _("Revision")
DATE_LABEL = _("Date")
AUTHOR_LABEL = _("Author")

def revision_grapher(history):
    """
    Expects a list of revision items like so:
    [
        item.commit = "..."
        item.parents = ["...", "..."]
    ]
    
    Output can be put directly into the CellRendererGraph
    """
    items = []
    revisions = []
    last_lines = []
    color = "#d3b9d3"
    for item in history:
        commit = unicode(item.revision)
        parents = []
        for parent in item.parents:
            parents.append(unicode(parent))

        if commit not in revisions:
            revisions.append(commit)

        index = revisions.index(commit)
        next_revisions = revisions[:]
        
        parents_to_add = []
        for parent in parents:
            if parent not in next_revisions:
                parents_to_add.append(parent)
        
        next_revisions[index:index+1] = parents_to_add

        lines = []
        for i, revision in enumerate(revisions):
            if revision in next_revisions:
                lines.append((i, next_revisions.index(revision), color))
            elif revision == commit:
                for parent in parents:
                    lines.append((i, next_revisions.index(parent), color))

        node = (index, "#a9f9d2")

        items.append((item, node, last_lines, lines))
        revisions = next_revisions
        last_lines = lines

    return items

class Log(InterfaceView):
    """
    Provides an interface to the Log UI
    
    """

    selected_rows = []
    selected_row = []
    paths_selected_rows = []
    display_items = []

    limit = 100

    def __init__(self, path):
        """
        @type   path: string
        @param  path: A path for which to get log items
        
        """
        
        InterfaceView.__init__(self, "log", "Log")

        self.get_widget("Log").set_title(_("Log - %s") % path)
        self.vcs = rabbitvcs.vcs.VCS()

        self.filter_text = None
        self.path = path
        self.cache = LogCache()

        self.rev_first = None
        self.rev_start = None
        self.rev_end = None
        self.rev_max = 1
        self.previous_starts = []
        self.initialize_revision_labels()
        self.revision_number_column = 0
        self.head_row = 0
        self.get_widget("limit").set_text(str(self.limit))

        self.message = rabbitvcs.ui.widget.TextView(
            self.get_widget("message")
        )

        self.stop_on_copy = False
        self.revision_clipboard = gtk.Clipboard()

    #
    # UI Signal Callback Methods
    #

    def on_destroy(self, widget, data=None):
        self.destroy()

    def on_close_clicked(self, widget, data=None):
        if self.is_loading:
            self.action.set_cancel(True)
            self.action.stop()
            self.set_loading(False)

        self.close()
    
    def on_key_pressed(self, widget, data):
        InterfaceView.on_key_pressed(self, widget, data)
        if (data.state & gtk.gdk.CONTROL_MASK and
            gtk.gdk.keyval_name(data.keyval).lower() == "c"):
            if len(self.revisions_table.get_selected_rows()) > 0:
                self.copy_revision_text()

    def on_stop_on_copy_toggled(self, widget):
        self.stop_on_copy = self.get_widget("stop_on_copy").get_active()
        if not self.is_loading:
            self.refresh()
    
    def on_refresh_clicked(self, widget):
        self.limit = int(self.get_widget("limit").get_text())
        self.cache.empty()
        self.load()

    
    def on_search(self, widget):
        tb = self.get_widget("search_buffer")
        self.filter_text = tb.get_text(tb.get_start_iter(), tb.get_end_iter()).lower()

        self.refresh()

    #
    # Revisions table callbacks
    #

    # In this UI, we have an ability to filter and display only certain items.
    def get_displayed_row_items(self, col):
        items = []
        for row in self.selected_rows:
            items.append(self.display_items[row][col])
        
        return items

    def on_revisions_table_row_activated(self, treeview, event, col):
        paths = self.revisions_table.get_displayed_row_items(1)

        rabbitvcs.util.helper.launch_diff_tool(*paths)

    def on_revisions_table_mouse_event(self, treeview, data=None):
        if len(self.revisions_table.get_selected_rows()) == 0:
            self.message.set_text("")
            self.paths_table.clear()
            return

        if data is not None and data.button == 3:
            self.show_revisions_table_popup_menu(treeview, data)

        self.paths_table.clear()
        self.message.set_text("")

        self.update_revision_message()             


    #
    # Paths table callbacks
    #

    def on_paths_table_row_activated(self, treeview, data=None, col=None):
        try:
            revision1 = unicode(self.display_items[self.revisions_table.get_selected_rows()[0]].revision)
            revision2 = unicode(self.display_items[self.revisions_table.get_selected_rows()[0]+1].revision)
            path_item = self.paths_table.get_row(self.paths_table.get_selected_rows()[0])[1]
            url = self.root_url + path_item
            self.view_diff_for_path(url, unicode(revision1), unicode(revision2), sidebyside=True)
        except IndexError:
            pass

    def on_paths_table_mouse_event(self, treeview, data=None):
        if data is not None and data.button == 3:
            self.show_paths_table_popup_menu(treeview, data)
    
    def show_paths_table_popup_menu(self, treeview, data):
        revisions = []
        for row in self.revisions_table.get_selected_rows():
            line = {
                "revision": self.display_items[row].revision,
                "author": self.display_items[row].author,
                "message": self.display_items[row].message
            }

            if self.display_items[row].parents:
                line["parents"] = self.display_items[row].parents

            try:
                line["next_revision"] = self.display_items[row+1].revision
            except IndexError,e:
                pass

            try:
                line["previous_revision"] = self.display_items[row-1].revision
            except IndexError,e:
                pass          
                      
            revisions.append(line)
        
        revisions.reverse()
        
        paths = []
        for row in self.paths_table.get_selected_rows():
            paths.append(self.paths_table.get_row(row)[1])
        
        LogBottomContextMenu(self, data, paths, revisions).show()
    
    #
    # Helper methods
    #

    def load_or_refresh(self):
        if self.cache.has(self.rev_start):
            self.refresh()
        else:
            self.load()
          
    def get_selected_revision_numbers(self):
        if len(self.revisions_table.get_selected_rows()) == 0:
            return ""

        revisions = []
        for row in self.revisions_table.get_selected_rows():
            revisions.append(int(self.revisions_table.get_row(row)[self.revision_number_column]))

        revisions.sort()
        return rabbitvcs.util.helper.encode_revisions(revisions)

    def get_selected_revision_number(self):
        if len(self.revisions_table.get_selected_rows()):
            return self.revisions_table.get_row(self.revisions_table.get_selected_rows()[0])[self.revision_number_column]
        else:
            return ""
    
    def set_start_revision(self, rev):
        self.get_widget("start").set_text(str(rev))

    def set_end_revision(self, rev):
        self.get_widget("end").set_text(str(rev))

    def initialize_revision_labels(self):
        self.set_start_revision(_("N/A"))
        self.set_end_revision(_("N/A"))

    def set_loading(self, loading):
        self.is_loading = loading

    def show_revisions_table_popup_menu(self, treeview, data):
        revisions = []
        for row in self.revisions_table.get_selected_rows():
            line = {
                "revision": self.display_items[row].revision,
                "author": self.display_items[row].author,
                "message": self.display_items[row].message
            }
            try:
                line["next_revision"] = self.display_items[row+1].revision
            except IndexError,e:
                pass

            try:
                line["previous_revision"] = self.display_items[row-1].revision
            except IndexError,e:
                pass          
                      
            revisions.append(line)
            
        LogTopContextMenu(self, data, self.path, revisions).show()

    #
    # Other helper methods
    #

    def view_diff_for_path(self, url, revision1, revision2=None, sidebyside=False):
        if revision2 == None:
            revision2 = revision1

        options = [
            "%s@%s" % (url, revision2),
            "%s@%s" % (url, revision1),
            "--vcs=%s" % self.get_vcs_name()
        ]
        
        if sidebyside:
            options += ["-s"] 

        rabbitvcs.util.helper.launch_ui_window("diff", options)

    def get_vcs_name(self):
        vcs = rabbitvcs.vcs.VCS_DUMMY
        if hasattr(self, "svn"):
            vcs = rabbitvcs.vcs.VCS_SVN
        elif hasattr(self, "git"):
            vcs = rabbitvcs.vcs.VCS_GIT

        return vcs

class SVNLog(Log):
    def __init__(self, path, merge_candidate_revisions=None):
        Log.__init__(self, path)
                
        self.svn = self.vcs.svn()
        self.merge_candidate_revisions = merge_candidate_revisions

        self.revisions_table = rabbitvcs.ui.widget.Table(
            self.get_widget("revisions_table"),
            [gobject.TYPE_STRING, gobject.TYPE_STRING, 
                gobject.TYPE_STRING, gobject.TYPE_STRING,
                rabbitvcs.ui.widget.TYPE_HIDDEN],
            [_("Revision"), _("Author"), 
                _("Date"), _("Message"),
                _("Color")],
            callbacks={
                "mouse-event":   self.on_revisions_table_mouse_event
            }
        )

        for i in range(4):
            column = self.revisions_table.get_column(i)
            cell = column.get_cell_renderers()[0]
            column.add_attribute(cell, 'foreground', 4)

        self.paths_table = rabbitvcs.ui.widget.Table(
            self.get_widget("paths_table"),
            [gobject.TYPE_STRING, gobject.TYPE_STRING, 
                gobject.TYPE_STRING, gobject.TYPE_STRING], 
            [_("Action"), _("Path"), 
                _("Copy From Path"), _("Copy From Revision")],
            callbacks={
                "mouse-event":      self.on_paths_table_mouse_event,
                "row-activated":    self.on_paths_table_row_activated
            },
            flags={
                "sortable": True, 
                "sort_on": 1
            }
        )

        self.initialize_root_url()
        self.load_or_refresh()

    def initialize_root_url(self):
        action = SVNAction(
            self.svn,
            notification=False,
            run_in_thread=False
        )
        
        self.root_url = action.run_single(
            self.svn.get_repo_root_url,
            self.path
        )

    #
    # Log-loading callback methods
    #
    
    def refresh(self):
        """
        Refresh the items in the main log table that shows Revision/Author/etc.
        
        """
        
        self.revision_items = []
        self.revisions_table.clear()
        self.message.set_text("")
        self.paths_table.clear()
        
        if self.rev_start and self.cache.has(self.rev_start):
            self.revision_items = self.cache.get(self.rev_start)
        else:
            # Make sure the int passed is the order the log call was made
            self.revision_items = self.action.get_result(0)

        if not self.revision_items or len(self.revision_items) == 0:
            return
        
        # Get the starting/ending point from the actual returned revisions
        self.rev_start = unicode(self.revision_items[0].revision)
        self.rev_end = unicode(self.revision_items[-1].revision)

        if not self.rev_first:
            self.rev_first = self.rev_start
        
        self.cache.set(self.rev_start, self.revision_items)
        
        # The first time the log items return, the rev_start will be as large
        # as it will ever be.  So set this to our maximum revision.
        if self.rev_start > self.rev_max:
            self.rev_max = self.rev_start
        
        self.display_items = []

        for item in self.revision_items:
            msg = cgi.escape(item.message).lower()

            should_add = not self.filter_text
            should_add = should_add or msg.find(self.filter_text) > -1
            should_add = should_add or item.author.lower().find(self.filter_text) > -1
            should_add = should_add or str(item.revision).lower().find(self.filter_text) > -1
            should_add = should_add or str(item.date).lower().find(self.filter_text) > -1

            if should_add:
                self.display_items.append(item)

        self.set_start_revision(self.rev_start)
        self.set_end_revision(self.rev_end)

        self.check_previous_sensitive()
        self.check_next_sensitive()

        for item in self.display_items:
            msg = cgi.escape(rabbitvcs.util.helper.format_long_text(item.message, 80))
            rev = item.revision
            color = "#000000"
            if (self.merge_candidate_revisions != None and
                int(rev.short()) not in self.merge_candidate_revisions):
                color = "#c9c9c9"

            self.populate_table(rev, item.author, item.date, msg, color)

            # Stop on copy after adding the item to the table
            # so the user can look at the item that was copied
            if self.stop_on_copy:
                for path in item.changed_paths:
                    if path.copy_from_path or path.copy_from_revision:
                        self.set_loading(False)
                        return

        self.set_loading(False)

    def populate_table(self, revision, author, date, msg, color):
        self.revisions_table.append([
            unicode(revision),
            author,
            rabbitvcs.util.helper.format_datetime(date),
            msg,
            color
        ])


    def load(self):
        self.set_loading(True)

        self.action = SVNAction(
            self.svn,
            notification=False
        )        

        start = self.svn.revision("head")
        if self.rev_start:
            start = self.svn.revision("number", number=self.rev_start)

        self.action.append(
            self.svn.log, 
            self.path,
            revision_start=start,
            limit=self.limit,
            discover_changed_paths=True
        )
        self.action.append(self.refresh)
        self.action.start()

    def edit_revprop(self, prop_name, prop_value, callback=None):

        failure = False
        url = self.svn.get_repo_url(self.path)

        self.action = SVNAction(
            self.svn,
            notification=False
        )

        for row in self.revisions_table.get_selected_rows():
            item = self.display_items[row]
            self.action.append(
                self.svn.revpropset,
                prop_name,
                prop_value,
                url,
                item.revision
            )
            
            callback(row, prop_value)
        
        self.action.start()

    def on_log_message_edited(self, index, val):
        self.display_items[index].message = val
        self.revisions_table.set_row_item(index, 3, val)
        self.message.set_text(val)

    def on_author_edited(self, index, val):
        self.display_items[index].author = val
        self.revisions_table.set_row_item(index, 1, val)

    def copy_revision_text(self):
        text = "" 
        for selected_row in self.revisions_table.get_selected_rows():
            item = self.display_items[selected_row]
            
            text += "%s: %s\n" % (REVISION_LABEL, unicode(item.revision))
            text += "%s: %s\n" % (AUTHOR_LABEL, unicode(item.author))
            text += "%s: %s\n" % (DATE_LABEL, unicode(item.date))
            text += "%s\n\n"   % item.message
            if item.changed_paths is not None:
                for subitem in item.changed_paths:
                    text += "%s\t%s" % (subitem.action, subitem.path)

                    if subitem.copy_from_path or subitem.copy_from_revision:
                        text += " (Copied from %s %s)" % (subitem.copy_from_path, subitem.copy_from_revision)

                    text += "\n"

            text += "\n\n\n"

        self.revision_clipboard.set_text(text)

    def update_revision_message(self):
        combined_paths = []
        subitems = []
               
        for selected_row in self.revisions_table.get_selected_rows():
            item = self.display_items[selected_row]
            
            if len(self.revisions_table.get_selected_rows()) == 1:
                self.message.set_text(item.message)
            else:
                indented_message = item.message.replace("\n","\n\t")
                self.message.append_text(
					"%s %s:\n\t%s\n" % (REVISION_LABEL,
                                        unicode(item.revision),
                                        indented_message))
            if item.changed_paths is not None:
                for subitem in item.changed_paths:
                    if subitem.path not in combined_paths:
                        combined_paths.append(subitem.path)
                        
                        subitems.append([
                            subitem.action,
                            subitem.path,
                            subitem.copy_from_path,
                            subitem.copy_from_revision
                        ])

        subitems.sort(lambda x, y: cmp(x[1],y[1]))
        for subitem in subitems:
            self.paths_table.append([
                subitem[0],
                subitem[1],
                subitem[2],
                subitem[3]
            ])

    def on_previous_clicked(self, widget):
        self.rev_start = self.previous_starts.pop()
        self.load_or_refresh()
                    
    def on_next_clicked(self, widget):
        self.previous_starts.append(self.rev_start)
        self.rev_start = int(self.rev_end) - 1

        if self.rev_start < 1:
            self.rev_start = 1

        self.load_or_refresh()

    def check_previous_sensitive(self):
        sensitive = (self.rev_start < self.rev_max)
        self.get_widget("previous").set_sensitive(sensitive)

    def check_next_sensitive(self):
        sensitive = True
        if self.rev_end == 1:
            sensitive = False
        if len(self.revision_items) < self.limit:
            sensitive = False

        self.get_widget("next").set_sensitive(sensitive)

class GitLog(Log):
    def __init__(self, path):
        Log.__init__(self, path)
        
        self.git = self.vcs.git(path)
        self.limit = 500
        
        self.get_widget("stop_on_copy").hide()
        
        self.revision_number_column = 1

        self.revisions_table = rabbitvcs.ui.widget.Table(
            self.get_widget("revisions_table"),
            [rabbitvcs.ui.widget.TYPE_GRAPH, gobject.TYPE_STRING, 
                rabbitvcs.ui.widget.TYPE_MARKUP, 
                rabbitvcs.ui.widget.TYPE_MARKUP, rabbitvcs.ui.widget.TYPE_MARKUP], 
            [_("Graph"), _("Revision"), _("Author"), 
                _("Date"), _("Message")],
            filters=[{
                "callback": rabbitvcs.ui.widget.git_revision_filter,
                "user_data": {
                    "column": 1
                }
            }],
            callbacks={
                "mouse-event":   self.on_revisions_table_mouse_event
            }
        )

        self.paths_table = rabbitvcs.ui.widget.Table(
            self.get_widget("paths_table"),
            [gobject.TYPE_STRING, gobject.TYPE_STRING], 
            [_("Action"), _("Path")],
            callbacks={
                "mouse-event":      self.on_paths_table_mouse_event,
                "row-activated":    self.on_paths_table_row_activated
            },
            flags={
                "sortable": True, 
                "sort_on": 1
            }
        )
        self.start_point = 0
        self.initialize_root_url()
        self.load_or_refresh()

    #
    # Log-loading callback methods
    #
    
    def refresh(self):
        """
        Refresh the items in the main log table that shows Revision/Author/etc.
        
        """
        
        self.revisions_table.clear()
        self.message.set_text("")
        self.paths_table.clear()
        
        # Make sure the int passed is the order the log call was made
        self.revision_items = self.action.get_result(0)
        
        if not self.revision_items or len(self.revision_items) == 0:
            return
        
        self.set_start_revision(self.revision_items[0].revision.short())
        self.set_end_revision(self.revision_items[-1].revision.short())

        self.display_items = []

        for item in self.revision_items:
            msg = cgi.escape(item.message).lower()

            should_add = not self.filter_text
            should_add = should_add or msg.find(self.filter_text) > -1
            should_add = should_add or item.author.lower().find(self.filter_text) > -1
            should_add = should_add or unicode(item.revision).lower().find(self.filter_text) > -1
            should_add = should_add or str(item.date).lower().find(self.filter_text) > -1

            if should_add:
                self.display_items.append(item)

        grapher = revision_grapher(self.display_items)
        max_columns = 1
        for (item, node, in_lines, out_lines) in grapher:
            if max_columns < len(out_lines):
                max_columns = len(out_lines)

        if not self.filter_text:
            graph_column = self.revisions_table.get_column(0)
            cell = graph_column.get_cell_renderers()[0]
            self.revisions_table.set_column_width(0, 16*max_columns)

        index = 0
        for (item, node, in_lines, out_lines) in grapher:
            revision = unicode(item.revision)
            msg = cgi.escape(rabbitvcs.util.helper.format_long_text(item.message, 80))
            author = item.author
            date = rabbitvcs.util.helper.format_datetime(item.date)
            
            if item.head:
                self.head_row = index
                msg = "<b>%s</b>" % msg
                author = "<b>%s</b>" % author
                date = "<b>%s</b>" % date            
            
            graph_render = ()
            if not self.filter_text:
                graph_render = (node, in_lines, out_lines)

            self.revisions_table.append([
                graph_render,
                revision,
                author,
                date,
                msg
            ])

            index += 1

        self.check_previous_sensitive()
        self.check_next_sensitive()
        self.set_loading(False)
    
    def load(self):
        self.set_loading(True)

        self.action = GitAction(
            self.git,
            notification=False,
            run_in_thread=True
        )        

        self.action.append(
            self.git.log,
            path=self.path,
            skip=self.start_point,
            limit=self.limit+1
        )
        self.action.append(self.refresh)
        self.action.start()

    def copy_revision_text(self):
        text = ""
        for selected_row in self.revisions_table.get_selected_rows():
            item = self.display_items[selected_row]

            text += "%s: %s\n" % (REVISION_LABEL, unicode(item.revision.short()))
            text += "%s: %s\n" % (AUTHOR_LABEL, unicode(item.author))
            text += "%s: %s\n" % (DATE_LABEL, unicode(item.date))
            text += "%s\n\n" % item.message
            
        self.revision_clipboard.set_text(text)

    def update_revision_message(self):
        combined_paths = []
        subitems = []
        
        for selected_row in self.revisions_table.get_selected_rows():
            item = self.display_items[selected_row]

            if len(self.revisions_table.get_selected_rows()) == 1:
                self.message.set_text(item.message)
            else:
                indented_message = item.message.replace("\n","\n\t")
                self.message.append_text(
					"%s %s:\n\t%s\n" % (REVISION_LABEL,
                                        item.revision.short(),
                                        indented_message))

            for subitem in item.changed_paths:
                
                if subitem.path not in combined_paths:
                    combined_paths.append(subitem.path)
                    
                    subitems.append([
                        subitem.action,
                        subitem.path
                    ])

        subitems.sort(lambda x, y: cmp(x[1],y[1]))
        for subitem in subitems:
            self.paths_table.append([
                subitem[0],
                subitem[1]
            ])

    def on_previous_clicked(self, widget):
        self.start_point -= self.limit
        if self.start_point < 0:
            self.start_point = 0
        self.load_or_refresh()
                    
    def on_next_clicked(self, widget):
        self.start_point += self.limit
        self.load()

    def check_previous_sensitive(self):
        sensitive = (self.start_point > 0)
        self.get_widget("previous").set_sensitive(sensitive)

    def check_next_sensitive(self):
        sensitive = True
        if len(self.revision_items) < self.limit:
            sensitive = False

        self.get_widget("next").set_sensitive(sensitive)

    def initialize_root_url(self):
        self.root_url = self.git.get_repository() + "/"

class SVNLogDialog(SVNLog):
    def __init__(self, path, ok_callback=None, multiple=False, merge_candidate_revisions=None):
        """
        Override the normal SVNLog class so that we can hide the window as we need.
        Also, provide a callback for when the OK button is clicked so that we
        can get some desired data.
        """
        SVNLog.__init__(self, path, merge_candidate_revisions)
        self.ok_callback = ok_callback
        self.multiple = multiple
        
    def on_destroy(self, widget):
        pass
    
    def on_close_clicked(self, widget, data=None):
        self.hide()
        if self.ok_callback is not None:
            if self.multiple == True:
                self.ok_callback(self.get_selected_revision_numbers())
            else:
                self.ok_callback(self.get_selected_revision_number())

class GitLogDialog(GitLog):
    def __init__(self, path, ok_callback=None, multiple=False):
        """
        Override the normal GitLog class so that we can hide the window as we need.
        Also, provide a callback for when the OK button is clicked so that we
        can get some desired data.
        """
        GitLog.__init__(self, path)
        self.ok_callback = ok_callback
        self.multiple = multiple
        
    def on_destroy(self, widget):
        pass
    
    def on_close_clicked(self, widget, data=None):
        self.hide()
        if self.ok_callback is not None:
            if self.multiple == True:
                self.ok_callback(self.get_selected_revision_numbers())
            else:
                self.ok_callback(self.get_selected_revision_number())

class LogCache:
    def __init__(self, cache={}):
        self.cache = cache
    
    def set(self, key, val):
        self.cache[key] = val
    
    def get(self, key):
        return self.cache[key]
    
    def has(self, key):
        return (key in self.cache)
    
    def empty(self):
        self.cache = {}


class MenuViewDiffWorkingCopy(MenuItem):
    identifier = "RabbitVCS::View_Diff_Working_Copy"
    label = _("View diff against working copy")
    icon = "rabbitvcs-diff"

class MenuViewDiffPreviousRevision(MenuItem):
    identifier = "RabbitVCS::View_Diff_Previous_Revision"
    label = _("View diff against previous revision")
    icon = "rabbitvcs-diff"

class MenuViewDiffRevisions(MenuItem):
    identifier = "RabbitVCS::View_Diff_Revisions"
    label = _("View diff between revisions")
    icon = "rabbitvcs-diff"

class MenuCompareWorkingCopy(MenuItem):
    identifier = "RabbitVCS::Compare_Working_Copy"
    label = _("Compare with working copy")
    icon = "rabbitvcs-compare"

class MenuComparePreviousRevision(MenuItem):
    identifier = "RabbitVCS::Compare_Previous_Revision"
    label = _("Compare with previous revision")
    icon = "rabbitvcs-compare"

class MenuCompareRevisions(MenuItem):
    identifier = "RabbitVCS::Compare_Revisions"
    label = _("Compare revisions")
    icon = "rabbitvcs-compare"

class MenuShowChangesPreviousRevision(MenuItem):
    # This is for the revs list
    identifier = "RabbitVCS::Show_Changes_Previous_Revision"
    label = _("Show changes against previous revision")
    icon = "rabbitvcs-changes"
    
class MenuShowChangesRevisions(MenuItem):
    # This is for the revs list
    identifier = "RabbitVCS::Show_Changes_Revisions"
    label = _("Show changes between revisions")
    icon = "rabbitvcs-changes"

class MenuUpdateToThisRevision(MenuItem):
    identifier = "RabbitVCS::Update_To_This_Revision"
    label = _("Update to this revision")
    tooltip = _("Update the selected path to this revision")
    icon = "rabbitvcs-update"

class MenuRevertChangesFromThisRevision(MenuItem):
    identifier = "RabbitVCS::Revert_Changes_From_This_Revision"
    label = _("Revert changes from this revision")
    tooltip = _("Update the selected path by reverse merging the changes")
    icon = "rabbitvcs-revert"    

class MenuCopyClipboard(MenuItem):
    identifier = "RabbitVCS::Copy_Clipboard"
    label = _("Copy to clipboard")
    tooltip = _("Copy to clipboard the full data of these revisions")

class MenuEditAuthor(MenuItem):
    identifier = "RabbitVCS::Edit_Author"
    label = _("Edit author...")

class MenuEditAuthor(MenuItem):
    identifier = "RabbitVCS::Edit_Author"
    label = _("Edit author...")

class MenuEditLogMessage(MenuItem):
    identifier = "RabbitVCS::Edit_Log_Message"
    label = _("Edit log message...")

class MenuEditRevisionProperties(MenuItem):
    identifier = "RabbitVCS::Edit_Revision_Properties"
    label = _("Edit revision properties...")
    icon = gtk.STOCK_EDIT

class MenuSeparatorLast(MenuSeparator):
    identifier = "RabbitVCS::Separator_Last"

class LogTopContextMenuConditions:
    def __init__(self, caller, vcs, path, revisions):
        self.caller = caller
        self.vcs = vcs
        self.path = path
        self.revisions = revisions
        
        self.vcs_name = caller.get_vcs_name()
        
    def view_diff_working_copy(self, data=None):
        return (self.vcs.is_in_a_or_a_working_copy(self.path) and len(self.revisions) == 1)

    def copy_clipboard(self, data=None):
        return (len(self.revisions) > 0)

    def view_diff_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def view_diff_revisions(self, data=None):
        return (len(self.revisions) > 1)
        
    def compare_working_copy(self, data=None):
        return (self.vcs.is_in_a_or_a_working_copy(self.path) and len(self.revisions) == 1) 

    def compare_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def compare_revisions(self, data=None):
        return (len(self.revisions) > 1)

    def show_changes_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def show_changes_revisions(self, data=None):
        return (len(self.revisions) > 1)

    def update_to_this_revision(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_SVN and len(self.revisions) == 1)

    # TODO Evaluate multiple revisions later
    # TODO Git?
    def revert_changes_from_this_revision(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_SVN and len(self.revisions) == 1)

    def checkout(self, data=None):
        return (len(self.revisions) == 1)

    def branches(self, data=None):
        return (len(self.revisions) == 1 and self.vcs_name == rabbitvcs.vcs.VCS_GIT)

    def tags(self, data=None):
        return (len(self.revisions) == 1 and self.vcs_name == rabbitvcs.vcs.VCS_GIT)

    def branch_tag(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_SVN and len(self.revisions) == 1)

    def export(self, data=None):
        return (len(self.revisions) == 1)

    def edit_author(self, data=None):
        return self.vcs_name == rabbitvcs.vcs.VCS_SVN

    def edit_log_message(self, data=None):
        return self.vcs_name == rabbitvcs.vcs.VCS_SVN

    def edit_revision_properties(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_SVN and len(self.revisions) == 1)

    def separator(self, data=None):
        return True

    def separator_last(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_SVN)
        
    def merge(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_GIT)

    def reset(self, data=None):
        return (self.vcs_name == rabbitvcs.vcs.VCS_GIT)

class LogTopContextMenuCallbacks:
    def __init__(self, caller, vcs, path, revisions):
        self.caller = caller
        self.vcs = vcs
        self.path = path
        self.revisions = revisions

        self.vcs_name = self.caller.get_vcs_name()

    def find_parent(self, revision):
        if ("parents" in revision) and len(revision["parents"]) > 0:
            parent = unicode(revision["parents"][0])
        elif ("next_revision" in revision):
            parent = unicode(revision["next_revision"])
        else:
            parent = unicode(int(unicode(revision["revision"])) - 1)

        return parent
        
    def view_diff_working_copy(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("diff", [
            "%s@%s" % (self.path, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def copy_clipboard(self, widget, data=None):
        self.caller.copy_revision_text()

    def view_diff_previous_revision(self, widget, data=None):
        parent = self.find_parent(self.revisions[0])

        rabbitvcs.util.helper.launch_ui_window("diff", [
            "%s@%s" % (self.path, parent),
            "%s@%s" % (self.path, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def view_diff_revisions(self, widget, data=None):
        path_older = self.path
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            path_older = self.vcs.svn().get_repo_url(self.path)
    
        rabbitvcs.util.helper.launch_ui_window("diff", [
            "%s@%s" % (path_older, self.revisions[1]["revision"].value),
            "%s@%s" % (self.path, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def compare_working_copy(self, widget, data=None):
        path_older = self.path
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            path_older = self.vcs.svn().get_repo_url(self.path)
    
        rabbitvcs.util.helper.launch_ui_window("diff", [
            "-s",
            "%s@%s" % (path_older, unicode(self.revisions[0]["revision"])),
            "%s" % (self.path),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def compare_previous_revision(self, widget, data=None):
        parent = self.find_parent(self.revisions[0])

        rabbitvcs.util.helper.launch_ui_window("diff", [
            "-s",
            "%s@%s" % (self.path, parent),
            "%s@%s" % (self.path, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def compare_revisions(self, widget, data=None):
        path_older = self.path
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            path_older = self.vcs.svn().get_repo_url(self.path)

        rabbitvcs.util.helper.launch_ui_window("diff", [
            "-s",
            "%s@%s" % (path_older, self.revisions[1]["revision"].value),
            "%s@%s" % (self.path, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def show_changes_previous_revision(self, widget, data=None):
        rev_first = unicode(self.revisions[0]["revision"])
        parent = self.find_parent(self.revisions[0])
        
        path = self.path
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            path = self.vcs.svn().get_repo_url(self.path)

        rabbitvcs.util.helper.launch_ui_window("changes", [
            "%s@%s" % (path, parent),
            "%s@%s" % (path, unicode(rev_first)),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def show_changes_revisions(self, widget, data=None):
        rev_first = unicode(self.revisions[0]["revision"])
        rev_last = unicode(self.revisions[0]["next_revision"])

        path = self.path
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            path = self.vcs.svn().get_repo_url(self.path)

        rabbitvcs.util.helper.launch_ui_window("changes", [
            "%s@%s" % (path, unicode(rev_first)),
            "%s@%s" % (path, unicode(rev_last)),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def update_to_this_revision(self, widget, data=None):        
        rabbitvcs.util.helper.launch_ui_window("updateto", [
            self.path,
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def revert_changes_from_this_revision(self, widget, data=None):        
        rabbitvcs.util.helper.launch_ui_window("merge", [
            self.path,
            unicode(self.revisions[0]["revision"]) + "-" + str(int(unicode(self.revisions[0]["revision"])) - 1),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])        
        
    def checkout(self, widget, data=None):
        url = ""
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            url = self.vcs.svn().get_repo_url(self.path)

        rabbitvcs.util.helper.launch_ui_window("checkout", [
            self.path, 
            url, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def branch_tag(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("branch", [
            self.path, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def branches(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("branches", [
            self.path, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def tags(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("tags", [
            self.path, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])
        
    def export(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("export", [
            self.path, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def merge(self, widget, data=None):
        extra = []
        if self.vcs_name == rabbitvcs.vcs.VCS_GIT:
            extra.append(unicode(self.revisions[0]["revision"]))
            try:
                fromrev = unicode(self.revisions[1]["revision"])
                extra.append(fromrev)
            except IndexError, e:
                pass
        
        extra += ["--vcs=%s" % self.caller.get_vcs_name()]
        
        rabbitvcs.util.helper.launch_ui_window("merge", [self.path] + extra)

    def reset(self, widget, data=None):
        rabbitvcs.util.helper.launch_ui_window("reset", [
            self.path, 
            "-r", unicode(self.revisions[0]["revision"]),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

    def edit_author(self, widget, data=None):
        message = ""
        if len(self.revisions) == 1:
            author = self.revisions[0]["author"]

        from rabbitvcs.ui.dialog import TextChange
        dialog = TextChange(_("Edit author"), author)
        (result, new_author) = dialog.run()

        if result == gtk.RESPONSE_OK:
            self.caller.edit_revprop("svn:author", new_author, self.caller.on_author_edited)

    def edit_log_message(self, widget, data=None):
        message = ""
        if len(self.revisions) == 1:
            message = self.revisions[0]["message"]

        from rabbitvcs.ui.dialog import TextChange
        dialog = TextChange(_("Edit log message"), message)
        (result, new_message) = dialog.run()

        if result == gtk.RESPONSE_OK:
            self.caller.edit_revprop("svn:log", new_message, self.caller.on_log_message_edited)

    def edit_revision_properties(self, widget, data=None):
        url = self.vcs.svn().get_repo_url(self.path)
        
        rabbitvcs.util.helper.launch_ui_window("revprops", [
            "%s@%s" % (url, unicode(self.revisions[0]["revision"])),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])

class LogTopContextMenu:
    """
    Defines context menu items for a table with files
    
    """
    def __init__(self, caller, event, path, revisions=[]):
        """    
        @param  caller: The calling object
        @type   caller: object
        
        @param  base_dir: The curent working directory
        @type   base_dir: string
        
        @param  path: The loaded path
        @type   path: string
        
        @param  revisions: The selected revisions
        @type   revisions: list of rabbitvcs.vcs.Revision object
        
        """        
        self.caller = caller
        self.event = event
        self.path = path
        self.revisions = revisions
        self.vcs = rabbitvcs.vcs.VCS()

        self.conditions = LogTopContextMenuConditions(
            self.caller,
            self.vcs, 
            self.path, 
            self.revisions
        )
        
        self.callbacks = LogTopContextMenuCallbacks(
            self.caller,
            self.vcs, 
            self.path,
            self.revisions
        )

        # The first element of each tuple is a key that matches a
        # ContextMenuItems item.  The second element is either None when there
        # is no submenu, or a recursive list of tuples for desired submenus.
        self.structure = [      
            (MenuViewDiffWorkingCopy, None),
            (MenuViewDiffPreviousRevision, None),
            (MenuViewDiffRevisions, None),
            (MenuCompareWorkingCopy, None),
            (MenuComparePreviousRevision, None),
            (MenuCompareRevisions, None),
            (MenuShowChangesPreviousRevision, None),
            (MenuShowChangesRevisions, None),
            (MenuSeparator, None),
            (MenuCopyClipboard, None),
            (MenuSeparator, None),
            (MenuUpdateToThisRevision, None),
            (MenuRevertChangesFromThisRevision, None),
            (MenuCheckout, None),
            (MenuBranches, None),
            (MenuTags, None),
            (MenuBranchTag, None),
            (MenuExport, None),
            (MenuMerge, None),
            (MenuReset, None),
            (MenuSeparatorLast, None),
            (MenuEditAuthor, None),
            (MenuEditLogMessage, None),
            (MenuEditRevisionProperties, None)
        ]
        
    def show(self):
        if len(self.revisions) == 0:
            return

        context_menu = GtkContextMenu(self.structure, self.conditions, self.callbacks)
        context_menu.show(self.event)


class LogBottomContextMenuConditions:
    def __init__(self, caller, vcs, paths, revisions):
        self.caller = caller
        self.vcs = vcs
        self.paths = paths
        self.revisions = revisions

    def view_diff_working_copy(self, data=None):
        return False

    def view_diff_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def view_diff_revisions(self, data=None):
        return (len(self.paths) == 1 and len(self.revisions) > 1)

    def compare_working_copy(self, data=None):
        return False

    def compare_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def compare_revisions(self, data=None):
        return (len(self.paths) == 1 and len(self.revisions) > 1)

    def show_changes_previous_revision(self, data=None):
        item = self.revisions[0]["revision"]
        return ("previous_revision" in self.revisions[0] and len(self.revisions) == 1)

    def show_changes_revisions(self, data=None):
        return (len(self.paths) == 1 and len(self.revisions) > 1)

    def _open(self, data=None):
        return True

    def annotate(self, data=None):
        return (len(self.paths) == 1)

    def separator(self, data=None):
        return True

class LogBottomContextMenuCallbacks:
    def __init__(self, caller, vcs, paths, revisions):
        self.caller = caller
        self.vcs = vcs
        self.svn = self.vcs.svn()
        self.vcs_name = self.caller.get_vcs_name()
            
        self.paths = paths
        self.revisions = revisions

    def find_parent(self, revision):
        if ("parents" in revision) and len(revision["parents"]) > 0:
            parent = unicode(revision["parents"][0])
        elif ("next_revision" in revision):
            parent = unicode(revision["next_revision"])
        else:
            parent = unicode(int(unicode(revision["revision"])) - 1)

        return parent

    def view_diff_previous_revision(self, widget, data=None):
        rev = unicode(self.revisions[0]["revision"])

        parent = self.find_parent(self.revisions[0])

        path_item = self.paths[0]
        url = self.caller.root_url + path_item
        self.caller.view_diff_for_path(url, rev, parent)

    def view_diff_revisions(self, widget, data=None):
        rev_first = unicode(self.revisions[0]["revision"])
        rev_last = unicode(self.revisions[-1]["revision"])
        path_item = self.paths[0]
        url = self.caller.root_url + path_item
        self.caller.view_diff_for_path(url, latest_revision_number=rev_last,
                                       earliest_revision_number=rev_first)

    def compare_previous_revision(self, widget, data=None):
        rev = unicode(self.revisions[0]["revision"])
        
        parent = self.find_parent(self.revisions[0])
            
        path_item = self.paths[0]
        url = self.caller.root_url + path_item
        self.caller.view_diff_for_path(url, rev, parent, sidebyside=True)
    
    def compare_revisions(self, widget, data=None):
        earliest_rev = unicode(self.revisions[0]["revision"])
        latest_rev = unicode(self.revisions[-1]["revision"])
        path_item = self.paths[0]
        url = self.caller.root_url + path_item
        self.caller.view_diff_for_path(url,
                                        latest_rev,
                                        sidebyside=True,
                                        earliest_revision_number=earliest_rev)

    def show_changes_previous_revision(self, widget, data=None):
        rev_first = unicode(self.revisions[0]["revision"])
        
        parent = self.find_parent(self.revisions[0])

        url = self.paths[0]
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            url = self.caller.root_url + self.paths[0]

        rabbitvcs.util.helper.launch_ui_window("changes", [
            "%s@%s" % (url, parent),
            "%s@%s" % (url, rev_last),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])
    
    def show_changes_revisions(self, widget, data=None):
        rev_first = unicode(self.revisions[0]["revision"])
        rev_last = unicode(self.revisions[-1]["revision"])

        url = self.paths[0]
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            url = self.caller.root_url + self.paths[0]
        
        rabbitvcs.util.helper.launch_ui_window("changes", [
            "%s@%s" % (url, rev_first),
            "%s@%s" % (url, rev_last),
            "--vcs=%s" % self.caller.get_vcs_name()
        ])
        

    def _open(self, widget, data=None):
        for path in self.paths:
            path = self.caller.root_url + path
            rabbitvcs.util.helper.launch_ui_window("open", [
                path, 
                "--vcs=%s" % self.vcs_name, 
                "-r", unicode(self.revisions[0]["revision"])
            ])

    def annotate(self, widget, data=None):
        url = self.paths[0]
        if self.vcs_name == rabbitvcs.vcs.VCS_SVN:
            url = self.caller.root_url + self.paths[0]

        rabbitvcs.util.helper.launch_ui_window("annotate", [
            url, 
            "--vcs=%s" % self.vcs_name, 
            "-r", unicode(self.revisions[0]["revision"])
        ])

class LogBottomContextMenu:
    """
    Defines context menu items for a table with files
    
    """
    def __init__(self, caller, event, paths, revisions):
        """    
        @param  caller: The calling object
        @type   caller: object
        
        @param  base_dir: The curent working directory
        @type   base_dir: string
        
        @param  paths: The selected paths
        @type   paths: list

        @param  revision: The selected revision
        @type   revision: rabbitvcs.vcs.Revision object
        
        """        
        self.caller = caller
        self.event = event
        self.paths = paths
        self.revisions = revisions
        self.vcs = rabbitvcs.vcs.VCS()
        self.svn = self.vcs.svn()

        self.conditions = LogBottomContextMenuConditions(
            self.caller,
            self.vcs, 
            self.paths, 
            self.revisions
        )
        
        self.callbacks = LogBottomContextMenuCallbacks(
            self.caller,
            self.vcs, 
            self.paths,
            self.revisions
        )

        # The first element of each tuple is a key that matches a
        # ContextMenuItems item.  The second element is either None when there
        # is no submenu, or a recursive list of tuples for desired submenus.
        self.structure = [
            (MenuViewDiffRevisions, None),
            (MenuViewDiffPreviousRevision, None),
            (MenuComparePreviousRevision, None),
            (MenuCompareRevisions, None),
            (MenuShowChangesPreviousRevision, None),
            (MenuShowChangesRevisions, None),
            (MenuSeparator, None),
            (MenuOpen, None),
            (MenuAnnotate, None)
        ]
        
    def show(self):
        if len(self.paths) == 0:
            return

        context_menu = GtkContextMenu(self.structure, self.conditions, self.callbacks)
        context_menu.show(self.event)

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNLog,
    rabbitvcs.vcs.VCS_GIT: GitLog
}

dialogs_map = {
    rabbitvcs.vcs.VCS_SVN: SVNLogDialog,
    rabbitvcs.vcs.VCS_GIT: GitLogDialog
}

def log_factory(path, vcs):
    if not vcs:
        guess = rabbitvcs.vcs.guess(path)
        vcs = guess["vcs"]

    return classes_map[vcs](path)

def log_dialog_factory(path, ok_callback=None, multiple=False, vcs=None):
    if not vcs:
        guess = rabbitvcs.vcs.guess(path)
        vcs = guess["vcs"]
    
    return dialogs_map[vcs](path, ok_callback, multiple)

if __name__ == "__main__":
    from rabbitvcs.ui import main, VCS_OPT
    (options, paths) = main(
        [VCS_OPT],
        usage="Usage: rabbitvcs log [--vcs=svn|git] [url_or_path]"
    )

    window = log_factory(paths[0], vcs=options.vcs)
    window.register_gtk_quit()
    gtk.main()



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os.path
import urllib

import pygtk
import gobject
import gtk

from rabbitvcs.ui import InterfaceView
import rabbitvcs.ui.widget
import rabbitvcs.ui.dialog
import rabbitvcs.ui.action
import rabbitvcs.util.helper
import rabbitvcs.vcs
from rabbitvcs.ui.updateto import GitUpdateToRevision
from rabbitvcs import gettext
_ = gettext.gettext

class Checkout(InterfaceView):
    """
    Provides an interface to check out a working copy.
    
    Pass it the destination path.
    
    """

    def __init__(self, path=None, url=None, revision=None):
        InterfaceView.__init__(self, "checkout", "Checkout")
        
        self.path = path
        self.vcs = rabbitvcs.vcs.VCS()

        self.repositories = rabbitvcs.ui.widget.ComboBox(
            self.get_widget("repositories"), 
            rabbitvcs.util.helper.get_repository_paths()
        )
        
        # We must set a signal handler for the gtk.Entry inside the combobox
        # Because glade will not retain that information
        self.repositories.set_child_signal(
            "key-release-event", 
            self.on_repositories_key_released
        )

        self.destination = rabbitvcs.util.helper.get_user_path()
        if path is not None:
            self.destination = path
            self.get_widget("destination").set_text(path)

        if url is not None:
            self.repositories.set_child_text(url)
        
        self.complete = False
        
    #
    # UI Signal Callback Methods
    #

    def _parse_path(self, path):
        if path.startswith("file://"):
            path = urllib.unquote(path)
            path = path[7:]
        return path
            
    def _get_path(self):
        path = self._parse_path(self.get_widget("destination").get_text())
        return os.path.normpath(path)

    def on_file_chooser_clicked(self, widget, data=None):
        chooser = rabbitvcs.ui.dialog.FolderChooser()
        path = chooser.run()
        if path is not None:
            self.get_widget("destination").set_text(path)

    def on_repositories_key_released(self, widget, data, userdata=None):
        if gtk.gdk.keyval_name(data.keyval) == "Return":
            if self.complete:
                self.on_ok_clicked(widget)

    def on_destination_changed(self, widget, data=None):
        self.check_form()

    def on_destination_key_released(self, widget, data):
        if gtk.gdk.keyval_name(data.keyval) == "Return":
            if self.complete:
                self.on_ok_clicked(widget)

    def on_repo_chooser_clicked(self, widget, data=None):
        from rabbitvcs.ui.browser import SVNBrowserDialog
        SVNBrowserDialog(self.repositories.get_active_text(), 
            callback=self.on_repo_chooser_closed)

    def on_repo_chooser_closed(self, new_url):
        self.repositories.set_child_text(new_url)
        self.check_form()

    def check_form(self):
        self.complete = True
        if self.repositories.get_active_text() == "":
            self.complete = False
        if self.get_widget("destination").get_text() == "":
            self.complete = False
        
        self.get_widget("ok").set_sensitive(self.complete)


class SVNCheckout(Checkout):
    def __init__(self, path=None, url=None, revision=None):
        Checkout.__init__(self, path, url, revision)
        self.get_widget("Checkout").set_title(_("Checkout - %s") % path)

        self.svn = self.vcs.svn()

        self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
            self.get_widget("revision_container"),
            self.svn,
            revision=revision,
            url_combobox=self.repositories,
            expand=True
        )

        self.get_widget("options_box").show()
        self.get_widget("revision_selector_box").show()

        self.check_form()

    def on_ok_clicked(self, widget):
        url = self.repositories.get_active_text()
        path = self._get_path()
        omit_externals = self.get_widget("omit_externals").get_active()
        recursive = self.get_widget("recursive").get_active()
        
        if not url or not path:
            rabbitvcs.ui.dialog.MessageBox(_("The repository URL and destination path are both required fields."))
            return
        
        revision = self.revision_selector.get_revision_object()
    
        self.hide()
        self.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            register_gtk_quit=self.gtk_quit_is_set()
        )
        self.action.append(self.action.set_header, _("Checkout"))
        self.action.append(self.action.set_status, _("Running Checkout Command..."))
        self.action.append(rabbitvcs.util.helper.save_repository_path, url)
        self.action.append(
            self.svn.checkout,
            rabbitvcs.util.helper.quote_url(url),
            path,
            recurse=recursive,
            revision=revision,
            ignore_externals=omit_externals
        )
        self.action.append(self.action.set_status, _("Completed Checkout"))
        self.action.append(self.action.finish)
        self.action.start()

    def on_repositories_changed(self, widget, data=None):
        # Do not use quoting for this bit
        url = self.repositories.get_active_text()
        tmp = url.replace("//", "/").split("/")[1:]
        append = ""
        prev = ""
        while len(tmp):
            prev = append
            append = tmp.pop()
            if append not in ("trunk", "branches", "tags"):
                break
                
            if append in ("http:", "https:", "file:", "svn:", "svn+ssh:"):
                append = ""
                break
        
        self.get_widget("destination").set_text(
            os.path.join(self.destination, append)
        )
        
        self.check_form()

class GitCheckout(GitUpdateToRevision):
    def __init__(self, path, url, revision):
        GitUpdateToRevision.__init__(self, path, revision)
        self.get_widget("Update").set_title(_("Checkout - %s") % path)
        self.get_widget("options_box").hide()

class GitCheckoutQuiet:
    def __init__(self, path):
        self.vcs = rabbitvcs.vcs.VCS()
        self.git = self.vcs.git(path)
        self.action = rabbitvcs.ui.action.GitAction(
            self.git,
            run_in_thread=False
        )

        self.action.append(self.git.checkout, [path])
        self.action.run()

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNCheckout,
    rabbitvcs.vcs.VCS_GIT: GitCheckout
}

def checkout_factory(vcs, path=None, url=None, revision=None, quiet=False):
    if not vcs:
        guess = rabbitvcs.vcs.guess(path)
        vcs = guess["vcs"]

    if vcs == rabbitvcs.vcs.VCS_DUMMY:
        return SVNCheckout(path, url, revision)
    elif vcs == rabbitvcs.vcs.VCS_GIT:
        if quiet:
            return GitCheckoutQuiet(path)
        else:
            return GitCheckout(path, url, revision)
    
    return classes_map[vcs](path, url, revision)

if __name__ == "__main__":
    from rabbitvcs.ui import main, REVISION_OPT, VCS_OPT, QUIET_OPT
    (options, args) = main(
        [REVISION_OPT, VCS_OPT, QUIET_OPT],
        usage="Usage: rabbitvcs checkout --vcs=[git|svn] [url] [path]"
    )
    
    # If two arguments are passed:
    #   The first argument is expected to be a url
    #   The second argument is expected to be a path
    # If one argument is passed:
    #   If the argument exists, it is a path
    #   Otherwise, it is a url
    path = url = None
    if len(args) == 2:
        path = args[0]
        url = args[1]
    elif len(args) == 1:
        if os.path.exists(args[0]):
            path = args[0]
        else:
            url = args[0]

    if options.quiet:
        window = checkout_factory(options.vcs, path=path, url=url, revision=options.revision, quiet=options.quiet)
    else:
        window = checkout_factory(options.vcs, path=path, url=url, revision=options.revision, quiet=options.quiet)
        window.register_gtk_quit()
        gtk.main()



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os.path
import thread

import pygtk
import gobject
import gtk
from datetime import datetime

from rabbitvcs.ui import InterfaceView
from rabbitvcs.util.contextmenu import GtkContextMenu, GtkContextMenuCaller, \
    GtkFilesContextMenuConditions
from rabbitvcs.util.contextmenuitems import *
import rabbitvcs.ui.widget
import rabbitvcs.ui.dialog
import rabbitvcs.ui.action
import rabbitvcs.util.helper
import rabbitvcs.vcs
from rabbitvcs.util.log import Log
from rabbitvcs.util.decorators import gtk_unsafe

log = Log("rabbitvcs.ui.browser")

from rabbitvcs import gettext
_ = gettext.gettext

gtk.gdk.threads_init()

class SVNBrowser(InterfaceView, GtkContextMenuCaller):
    def __init__(self, url):
        InterfaceView.__init__(self, "browser", "Browser")

        self.vcs = rabbitvcs.vcs.VCS()
        self.svn = self.vcs.svn()

        self.url = ""
        if self.svn.is_in_a_or_a_working_copy(url):
            action = rabbitvcs.ui.action.SVNAction(self.svn, notification=False, run_in_thread=False)
            self.url = action.run_single(self.svn.get_repo_url, url)
        elif self.svn.is_path_repository_url(url):
            self.url = url

        self.urls = rabbitvcs.ui.widget.ComboBox(
            self.get_widget("urls"), 
            rabbitvcs.util.helper.get_repository_paths()
        )
        if self.url:
            self.urls.set_child_text(rabbitvcs.util.helper.unquote_url(self.url))

        # We must set a signal handler for the gtk.Entry inside the combobox
        # Because glade will not retain that information
        self.urls.set_child_signal(
            "key-release-event", 
            self.on_urls_key_released
        )

        self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
            self.get_widget("revision_container"),
            self.svn,
            url_combobox=self.urls
        )

        self.items = []
        self.list_table = rabbitvcs.ui.widget.Table(
            self.get_widget("list"), 
            [rabbitvcs.ui.widget.TYPE_PATH, gobject.TYPE_INT, 
                gobject.TYPE_INT, gobject.TYPE_STRING, gobject.TYPE_FLOAT], 
            [_("Path"), _("Revision"), _("Size"), _("Author"), _("Date")],
            filters=[{
                "callback": self.file_filter,
                "user_data": {
                    "column": 0
                }
            },{
                "callback": self.revision_filter,
                "user_data": {
                    "column": 1
                }
            },{
                "callback": self.size_filter,
                "user_data": {
                    "column": 2
                }
            },{
                "callback": self.date_filter,
                "user_data": {
                    "column": 4
                }
            }],
            filter_types=[gobject.TYPE_STRING, gobject.TYPE_STRING, 
                gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING], 
            callbacks={
                "file-column-callback": self.file_column_callback,
                "row-activated": self.on_row_activated,
                "mouse-event":   self.on_list_table_mouse_event
            },
            flags={
                "sortable": True
            }
        )
        
        self.clipboard = None
        self.url_clipboard = gtk.Clipboard()
        self.repo_root_url = None

        if self.url:
            rabbitvcs.util.helper.save_repository_path(url)
            self.load()

    def load(self):
        self.url = self.urls.get_active_text()
        self.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        revision = self.revision_selector.get_revision_object()
        self.action.append(
                        self.svn.list,
                        rabbitvcs.util.helper.quote_url(self.url), 
                        revision=revision, recurse=False)
        self.action.append(self.init_repo_root_url)
        self.action.append(self.populate_table, 0)
        self.action.start()

    @gtk_unsafe
    def populate_table(self, item_index=0):
        self.list_table.clear()
        self.items = self.action.get_result(item_index)
        self.items.sort(self.sort_files)
        
        self.list_table.append(["..", 0, 0, "", 0])
        for item,locked in self.items[1:]:
            self.list_table.append([
                item.path,
                item.created_rev.number,
                item.size,
                item.last_author,
                item.time
            ])
    
    def init_repo_root_url(self):
        if self.repo_root_url is None and self.svn.is_in_a_or_a_working_copy(self.url):
            action = rabbitvcs.ui.action.SVNAction(self.svn, notification=False, run_in_thread=False)
            self.repo_root_url = action.run_single(self.svn.get_repo_root_url, self.url)

    def on_refresh_clicked(self, widget):
        rabbitvcs.util.helper.save_repository_path(self.urls.get_active_text())
        self.load()

    def on_row_activated(self, treeview, data, col):
        path = self.list_table.get_selected_row_items(0)[0]
        if path == "..":
            path = self.url.split("/")[0:-1]
            self.url = "/".join(path)
        else:
            self.url = path

        if self.file_column_callback(self.url) == "dir" or self.url != path:
            self.urls.set_child_text(rabbitvcs.util.helper.unquote_url(self.url))
            self.load()
        else:
            self._open([self.url])

    def on_urls_key_released(self, widget, data, userdata):
        if gtk.gdk.keyval_name(data.keyval) == "Return":
            rabbitvcs.util.helper.save_repository_path(self.urls.get_active_text())
            self.load()

    def file_column_callback(self, filename):
        """
        Determine the node kind (dir or file) from our retrieved items list
        """

        if filename == "..":
            return "dir"

        for item,locked in self.items:
            if item.path == filename:
                return self.svn.NODE_KINDS_REVERSE[item.kind]
        return None

    def sort_files(self, x, y):
        """
        Sort the browser listing so that folders are on top and then sort
        alphabetically.

        """
        xkind = self.svn.NODE_KINDS_REVERSE[x[0].kind]
        ykind = self.svn.NODE_KINDS_REVERSE[y[0].kind]
        if xkind == "dir" and ykind == "dir":
            return cmp(x[0].repos_path, y[0].repos_path)
        elif xkind == "dir" and ykind == "file":
            return -1
        else:
            return 1

    def file_filter(self, row, column, user_data=None):
        """
        Table filter to just show the basename of the item path
        """

        if row[column]:
            return os.path.basename(row[column])
            
        return row[column]

    def size_filter(self, row, column, user_data=None):
        """
        Table filter to convert the item size to a "pretty" filesize
        """

        if self.file_column_callback(row[0]) == "file":
            return rabbitvcs.util.helper.pretty_filesize(int(row[column]))

        return ""

    def revision_filter(self, row, column, user_data=None):
        """
        Table filter to convert revision to a desired format
        """
        
        if row[0] == "..":
            return ""
        
        return row[column]

    def date_filter(self, row, column, user_data=None):
        """
        Table filter to convert the item date to something readable
        """
        
        if row[0] == "..":
            return ""
        
        if row[column]:
            change_time = datetime.fromtimestamp(float(row[column]))
            return rabbitvcs.util.helper.format_datetime(change_time)
        
        return str(row[column])

    def on_list_table_mouse_event(self, treeview, data=None):
        if data is not None and data.button == 3:
            self.show_list_table_popup_menu(treeview, data)

    def show_list_table_popup_menu(self, treeview, data):
        paths = self.list_table.get_selected_row_items(0)
        if len(paths) == 0:
            paths.append(self.url)
        
        BrowserContextMenu(self, data, None, self.vcs, paths).show()

    def update_clipboard(self, action, urls):
        self.clipboard = {
            "action": action,
            "urls": urls
        }

    def clipboard_has_cut(self):
        return (self.clipboard is not None and self.clipboard["action"] == "cut")

    def clipboard_has_copy(self):
        return (self.clipboard is not None and self.clipboard["action"] == "cut")

    def empty_clipboard(self):
        self.clipboard = None
    
    def set_url_clipboard(self, url):
        self.url_clipboard.set_text(url)
    
    def get_repo_root_url(self):
        return self.repo_root_url

    def get_url(self):
        return self.urls.get_active_text()

    def _open(self, paths):
        self.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        
        exported_paths = []
        for path in paths:
            export_path = rabbitvcs.util.helper.get_tmp_path(os.path.basename(paths[0]))
            exported_paths.append(export_path)
            self.action.append(self.svn.export, paths[0], 
                export_path, revision=self.revision_selector.get_revision_object())

        for path in exported_paths:
            self.action.append(rabbitvcs.util.helper.open_item, path)

        self.action.start()

class SVNBrowserDialog(SVNBrowser):
    def __init__(self, path, callback=None):
        """
        Override the normal Browser class so that we can hide the window as we need.
        Also, provide a callback for when the close button is clicked so that we
        can get some desired data.
        """
        
        self.callback = callback

        gtk.stock_add([(gtk.STOCK_CLOSE, _("Select"), 0, 0, "")])
        
        SVNBrowser.__init__(self, path)
        
    def on_destroy(self, widget):
        pass
    
    def on_close_clicked(self, widget, data=None):
        self.hide()
        if self.callback is not None:
            path = self.urls.get_active_text()
            selected = self.list_table.get_selected_row_items(0)
            if len(selected) > 0:
                path = selected[0]
            self.callback(path)




class MenuCreateRepositoryFolder(MenuItem):
    identifier = "RabbitVCS::Create_Repository_Folder"
    label = _("Create folder...")
    icon = gtk.STOCK_NEW

class MenuBrowserCopyTo(MenuItem):
    identifier = "RabbitVCS::Browser_Copy_To"
    label = _("Copy to...")
    icon = gtk.STOCK_COPY

class MenuBrowserCopyUrlToClipboard(MenuItem):
    identifier = "RabbitVCS::Browser_Copy_Url_To_Clipboard"
    label = _("Copy URL to clipboard")
    icon = "rabbitvcs-asynchronous"

class MenuBrowserMoveTo(MenuItem):
    identifier = "RabbitVCS::Browser_Move_To"
    label = _("Move to...")
    icon = gtk.STOCK_SAVE_AS


class BrowserContextMenuConditions(GtkFilesContextMenuConditions):
    def __init__(self, vcs, paths, caller):
        GtkFilesContextMenuConditions.__init__(self, vcs, paths)
        self.caller = caller

    def _open(self, data1=None, data2=None):
        return True
    
    def show_log(self, data1=None, data2=None):
        return True
    
    def annotate(self, data1=None, data2=None):
        if self.path_dict["length"] == 1:
            return (self.caller.file_column_callback(self.paths[0]) == "file")

        return False
    
    def checkout(self, data1=None, data2=None):
        return True
    
    def export(self, data1=None, data2=None):
        return True
        
    def rename(self, data1=None):
        revision = self.caller.revision_selector.get_revision_object()
        return (revision.kind == "head")
    
    def delete(self, data1=None, data2=None):
        revision = self.caller.revision_selector.get_revision_object()
        return (revision.kind == "head")

    def create_repository_folder(self, data1=None):
        if self.path_dict["length"] == 1:
            return (self.caller.file_column_callback(self.paths[0]) == "dir")

        return (self.path_dict["length"] == 0)

    def browser_copy_to(self, data1=None, data2=None):
        return True

    def browser_copy_url_to_clipboard(self, data1=None, data2=None):
        return (self.path_dict["length"] == 1)

    def browser_move_to(self, data1=None, data2=None):
        revision = self.caller.revision_selector.get_revision_object()
        return (revision.kind == "head")

class BrowserContextMenuCallbacks:
    def __init__(self, caller, base_dir, vcs, paths=[]):
        self.caller = caller
        self.base_dir = base_dir
        self.vcs = vcs
        self.svn = self.vcs.svn()
        self.paths = paths
        self.guess = rabbitvcs.vcs.VCS_SVN

    def __update_browser_url(self, url):
        # Make sure the Browser variables are updated with the new path
        self.caller.urls.set_child_text(url)
        self.caller.url = url

    def __get_browser_revision(self):
        return self.caller.revision_selector.get_revision_object()

    def __generate_sources_list(self):
        # Generates a list of tuples where the first element is a path and the
        # second element is a primitive revision object
        # Used for the copy_to menu item
        sources = []
        for path in self.paths:
            sources.append((path,self.__get_browser_revision().primitive()))

        return sources

    def _open(self, data=None, user_data=None):
        self.caller._open(self.paths)
    
    def show_log(self, data=None, user_data=None):
        rabbitvcs.util.helper.launch_ui_window("log", ["--vcs=%s" % self.guess, self.paths[0]])
    
    def annotate(self, data=None, user_data=None):
        urlrev = self.paths[0]
        revision = self.__get_browser_revision()
        if revision.kind == "number":
            urlrev += "@" + revision.value
        rabbitvcs.util.helper.launch_ui_window("annotate", [urlrev])
    
    def checkout(self, data=None, user_data=None):
        args = [self.paths[0]]
        revision = self.__get_browser_revision()
        if revision.kind == "number":
            args = ["-r", revision.value] + args
        rabbitvcs.util.helper.launch_ui_window("checkout", args)
    
    def export(self, data=None, user_data=None):
        args = [self.paths[0]]
        revision = self.__get_browser_revision()
        if revision.kind == "number":
            args = ["-r", revision.value] + args
        rabbitvcs.util.helper.launch_ui_window("export", args)
        
    def rename(self, data=None, user_data=None):
        (base, filename) = os.path.split(self.paths[0])
    
        from rabbitvcs.ui.dialog import OneLineTextChange
        dialog = OneLineTextChange(_("Rename"), _("New Name:"), filename)
        (result, new_name) = dialog.run()
        
        if result == gtk.RESPONSE_CANCEL:
            return
        
        new_url = base.rstrip("/") + "/" + new_name
        path_to_refresh = self.caller.get_url()
        if self.paths[0] == path_to_refresh:
            path_to_refresh = new_url
            self.__update_browser_url(path_to_refresh)

        self.caller.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        self.caller.action.append(self.svn.move, self.paths[0], new_url)
        self.caller.action.append(self.svn.list, path_to_refresh, recurse=False)
        self.caller.action.append(self.caller.populate_table, 1)
        self.caller.action.start()
    
    def delete(self, data=None, user_data=None):
        path_to_refresh = self.caller.get_url() 
        if self.paths[0] == path_to_refresh:            
            # If the deleted path is the same as the current path, go to the parent
            path_to_refresh = path_to_refresh.split("/")[0:-1]
            path_to_refresh = "/".join(path_to_refresh)
        
            self.__update_browser_url(path_to_refresh)
        
        self.caller.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        self.caller.action.append(self.svn.remove, self.paths)
        self.caller.action.append(self.svn.list, path_to_refresh, recurse=False)
        self.caller.action.append(self.caller.populate_table, 1)
        self.caller.action.start()  

    def create_repository_folder(self, data=None, user_data=None):
        from rabbitvcs.ui.dialog import NewFolder
        dialog = NewFolder()
        result = dialog.run()
        if result is None:
            return
            
        (folder_name, log_message) = result
        new_url = self.paths[0].rstrip("/") + "/" + folder_name

        self.caller.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        self.caller.action.append(self.svn.mkdir, new_url, log_message)
        self.caller.action.append(self.svn.list, self.paths[0], recurse=False)
        self.caller.action.append(self.caller.populate_table, 1)
        self.caller.action.start()        

    def browser_copy_to(self, data=None, user_data=None):
        from rabbitvcs.ui.dialog import OneLineTextChange
        dialog = OneLineTextChange(
            _("Where do you want to copy the selection?"), 
            _("New Location:"),
            self.caller.get_url()
        )
        result = dialog.run()
        if result is None:
            return

        (response, new_url) = result
        if response == gtk.RESPONSE_CANCEL:
            return

        sources = self.__generate_sources_list()

        self.caller.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        self.caller.action.append(self.svn.copy_all, sources, new_url, copy_as_child=True)
        self.caller.action.append(self.svn.list, self.caller.get_url(), recurse=False)
        self.caller.action.append(self.caller.populate_table, 1)
        self.caller.action.start()

    def browser_copy_url_to_clipboard(self, data=None, user_data=None):
        self.caller.set_url_clipboard(self.paths[0])

    def browser_move_to(self, data=None, user_data=None):
        from rabbitvcs.ui.dialog import OneLineTextChange
        dialog = OneLineTextChange(
            _("Where do you want to move the selection?"), 
            _("New Location:"),
            self.caller.get_url()
        )
        result = dialog.run()
        if result is None:
            return

        (response, new_url) = result
        if response == gtk.RESPONSE_CANCEL:
            return

        self.caller.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            notification=False
        )
        self.caller.action.append(self.svn.move_all, self.paths, new_url, move_as_child=True)
        self.caller.action.append(self.svn.list, self.caller.get_url(), recurse=False)
        self.caller.action.append(self.caller.populate_table, 1)
        self.caller.action.start()

class BrowserContextMenu:
    def __init__(self, caller, event, base_dir, vcs, paths=[]):
        
        self.caller = caller
        self.event = event
        self.paths = paths
        self.base_dir = base_dir
        self.vcs = vcs
        self.svn = self.vcs.svn()
        
        self.conditions = BrowserContextMenuConditions(
            self.vcs, 
            paths,
            self.caller
        )
        self.callbacks = BrowserContextMenuCallbacks(
            self.caller, 
            self.base_dir,
            self.vcs, 
            paths
        )

        self.structure = [
            (MenuOpen, None),
            (MenuSeparator, None),
            (MenuShowLog, None),
            (MenuAnnotate, None),
            (MenuExport, None),
            (MenuCheckout, None),
            (MenuSeparator, None),
            (MenuCreateRepositoryFolder, None),
            (MenuSeparator, None),
            (MenuRename, None),
            (MenuDelete, None),
            (MenuBrowserCopyTo, None),
            (MenuBrowserCopyUrlToClipboard, None),
            (MenuBrowserMoveTo, None)
        ]

    def show(self):
        if len(self.paths) == 0:
            return

        context_menu = GtkContextMenu(self.structure, self.conditions, self.callbacks)
        context_menu.show(self.event)

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNBrowser,
    rabbitvcs.vcs.VCS_DUMMY: SVNBrowser
}

def browser_factory(path):
    guess = rabbitvcs.vcs.guess(path)
    return classes_map[guess["vcs"]](path)

if __name__ == "__main__":
    from rabbitvcs.ui import main
    (options, url) = main(
        usage="Usage: rabbitvcs browser [url]"
    )

    window = browser_factory(url[0])
    window.register_gtk_quit()
    gtk.main()



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

from os import getcwd
import os.path

import pygtk
import gobject
import gtk

from rabbitvcs.ui import InterfaceNonView
from rabbitvcs.ui.action import SVNAction, GitAction

import rabbitvcs.vcs

from rabbitvcs import gettext
_ = gettext.gettext

class SVNOpen(InterfaceNonView):
    """
    This class provides a handler to open tracked files.
    
    """

    def __init__(self, path, revision):
        """
        @type   path: string
        @param  path: The path to open
        
        @type   revision: string
        @param  revision: The revision of the file to open
        
        """
        
        InterfaceNonView.__init__(self)

        self.vcs = rabbitvcs.vcs.VCS()
        self.svn = self.vcs.svn()

        if revision and type(revision) in (str, unicode):
            revision_obj = self.svn.revision("number", number=revision)
        else:
            revision_obj = self.svn.revision("HEAD")

        url = path
        if not self.svn.is_path_repository_url(path):
            url = self.svn.get_repo_root_url(path) + '/' + path
        dest = rabbitvcs.util.helper.get_tmp_path("rabbitvcs-" + revision + "-" + os.path.basename(path))
        
        self.svn.export(
            url,
            dest,
            revision=revision_obj
        )
        
        rabbitvcs.util.helper.open_item(dest)

        raise SystemExit()

class GitOpen(InterfaceNonView):
    """
    This class provides a handler to open tracked files.
    
    """

    def __init__(self, path, revision):
        """
        @type   path: string
        @param  path: The path to open
        
        @type   revision: string
        @param  revision: The revision of the file to open
        
        """
        
        InterfaceNonView.__init__(self)

        self.vcs = rabbitvcs.vcs.VCS()
        self.git = self.vcs.git(path)

        if revision:
            revision_obj = self.git.revision(revision)
        else:
            revision_obj = self.git.revision("HEAD")

        dest_dir = rabbitvcs.util.helper.get_tmp_path("rabbitvcs-" + unicode(revision))
        
        self.git.export(
            path,
            dest_dir,
            revision=revision_obj
        )
        
        repo_path = self.git.find_repository_path(path)
        relative_path = path
        if path.startswith(repo_path):
            relative_path = path[len(repo_path)+1:]
        
        dest_path = "%s/%s" % (dest_dir, relative_path)
        
        rabbitvcs.util.helper.open_item(dest_path)

        raise SystemExit()

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNOpen,
    rabbitvcs.vcs.VCS_GIT: GitOpen
}

def open_factory(vcs, path, revision):
    if not vcs:
        guess = rabbitvcs.vcs.guess(path)
        vcs = guess["vcs"]
        
    return classes_map[vcs](path, revision)

if __name__ == "__main__":
    from rabbitvcs.ui import main, REVISION_OPT, VCS_OPT
    (options, paths) = main(
        [REVISION_OPT, VCS_OPT],
        usage="Usage: rabbitvcs open path [-r REVISION]"
    )
    
    window = open_factory(options.vcs, paths[0], options.revision)
    window.register_gtk_quit()
    gtk.main()



#
# Copyright (C) 2009 Jason Heeris <jason.heeris@gmail.com>
# Copyright (C) 2009 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2009 by Adam Plumb <adamplumb@gmail.com>#
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

""" Simple utility module for starting DBUS services.

This module contains helper functions for starting DBUS services. Usually they
would be used from within a constructor.
"""

import sys
import subprocess

import dbus

from rabbitvcs.util.log import Log
log = Log("rabbitvcs.services.service")

def start_service(script_file, dbus_service_name, dbus_object_path):
    """
    This function is used to start a service that exports a DBUS object. If the
    DBUS object can be found already, nothing is done and the function returns
    True. Otherwise we try to start the given script file and wait until we
    receive a newline over stdout. 
    
    The "wait for newline" mechanism ensures any function calling this one will
    not try to access the object via DBUS until it is ready. It is recommended
    to use something like
    
    glib.idle_add(sys.stdout.write, "Started service\n")
    glib.idle_add(sys.stdout.flush)
    mainloop.run()
 
    That way a newline will be sent when the mainloop is started.
    
    @param script_file: the Python script file to run if the DBUS object does
                        not already exist
    @type script_file: a Python script file that will create the DBUS object and
                       send a newline over stdout when it is ready
                       
    @param dbus_service_name: the name of the DBUS service to request
    @type dbus_service_name: string (confirming to the DBUS service format)

    @param dbus_object_path: the DBUS object path to request
    @type dbus_object_path: string (confirming to the DBUS object path format)
        
    @rtype: boolean
    @return: Whether or not the service was successfully started.
    """
    
    object_exists = False
    
    try:
        session_bus = dbus.SessionBus()
        obj = session_bus.get_object(dbus_service_name, dbus_object_path)
        object_exists = True
    except dbus.DBusException:
        proc = subprocess.Popen([sys.executable, script_file],
                               stdin=subprocess.PIPE,
                               stdout=subprocess.PIPE)
        pid = proc.pid
        log.debug("Started process: %i" % pid)
        
        # Wait for subprocess to send a newline, to tell us it's ready
        proc.stdout.readline() # We don't care what the message is
        object_exists = True
        
    return object_exists




#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

"""
Concrete VCS implementation for Mercurial functionality.
"""

import os.path
from datetime import datetime

from mercurial import commands, ui, hg

import rabbitvcs.util.helper

import rabbitvcs.vcs
import rabbitvcs.vcs.status
import rabbitvcs.vcs.log
import rabbitvcs.vcs.mercurial.util
from rabbitvcs.vcs.branch import BranchEntry
from rabbitvcs.util.log import Log

log = Log("rabbitvcs.vcs.mercurial")

from rabbitvcs import gettext
_ = gettext.gettext

class Revision:
    """
    Implements a simple revision object as a wrapper around the gittyup revision
    object.  This allows us to provide a standard interface to the object data.
    """

    def __init__(self, kind, value=None):
        self.kind = kind.upper()
        self.value = value
        
        if self.kind == "HEAD":
            self.value = "HEAD"
        
        self.is_revision_object = True

    def __unicode__(self):
        if self.value:
            return unicode(self.value)
        else:
            return self.kind
            
    def short(self):
        if self.value:
            return unicode(self.value)[0:7]
        else:
            return self.kind

    def __str__(self):
        return self.__unicode__()

    def __repr__(self):
        return self.__unicode__()

    def primitive(self):
        return self.value

class Mercurial:
    STATUS = {
        "normal":       "C",
        "added":        "A",
        "removed":      "R",
        "modified":     "M",
        "untracked":    "?",
        "missing":      "!"
    }
    
    STATUS_REVERSE = {
        "C":       "normal",
        "A":        "added",
        "R":      "removed",
        "M":     "modified",
        "?":    "untracked",
        "!":      "missing"
    }

    STATUSES_FOR_REVERT = [
        "missing",
        "modified",
        "removed"
    ]

    STATUSES_FOR_COMMIT = [
        "untracked",
        "missing",
        "modified",
        "added",
        "removed"
    ]
    
    STATUSES_FOR_STAGE = [
        "untracked"
    ]

    STATUSES_FOR_UNSTAGE = [
        "added"
    ]

    def __init__(self, repo=None):
        self.vcs = rabbitvcs.vcs.VCS_MERCURIAL
        self.interface = "mercurial"

        self.ui = ui.ui()
        self.repository = None
        if repo:
            self.repository_path = repo
            self.repository = hg.repository(self.ui, self.repository_path)

        self.cache = rabbitvcs.vcs.status.StatusCache()

    def set_repository(self, path):
        self.repository_path = path
        self.repository = hg.repository(self.ui, self.repository_path)

    def get_repository(self):
        return self.repository_path

    def find_repository_path(self, path):
        path_to_check = path
        while path_to_check != "/" and path_to_check != "":
            if os.path.isdir(os.path.join(path_to_check, ".hg")):
                return path_to_check
            
            path_to_check = os.path.split(path_to_check)[0]
        
        return None
    
    def get_relative_path(self, path):
        if path == self.repository_path:
            return ""

        return rabbitvcs.vcs.mercurial.util.relativepath(self.repository_path, path)

    def get_absolute_path(self, path):
        return os.path.join(self.repository_path, path).rstrip("/")
    
    def statuses(self, path, recurse=True, invalidate=False):
        mercurial_statuses = self.repository.status(clean=True, unknown=True)

        # the status method returns a series of tuples filled with files matching
        # the statuses below
        tuple_order = ["modified", "added", "removed", "missing", "unknown", "ignored", "clean"]
        
        # go through each tuple (each of which has a defined status), and
        # generate a flat list of rabbitvcs statuses
        statuses = []
        index = 0
        directories = {}
        for status_tuple in mercurial_statuses:
            content = tuple_order[index]
            for item in status_tuple:
                st_path = self.get_absolute_path(item)
                
                rabbitvcs_status = rabbitvcs.vcs.status.MercurialStatus({
                    "path": st_path,
                    "content": content
                })
                statuses.append(rabbitvcs_status)
                
                # determine the statuses of the parent folders
                dir_content = content
                if content in self.STATUSES_FOR_REVERT:
                    dir_content = "modified"
                
                path_to_check = os.path.dirname(st_path)
                while True:
                    if path_to_check not in directories or directories[path_to_check] not in self.STATUSES_FOR_COMMIT:
                        rabbitvcs_status = rabbitvcs.vcs.status.MercurialStatus({
                            "path": path_to_check,
                            "content": dir_content
                        })
                        statuses.append(rabbitvcs_status)
                        directories[path_to_check] = dir_content
                    
                    if path_to_check == "" or path_to_check == self.repository_path:
                        break

                    path_to_check = os.path.split(path_to_check)[0]

            index += 1

        return statuses
    
    def status(self, path, summarize=True, invalidate=False):
        all_statuses = self.statuses(path, invalidate=invalidate)

        if summarize:
            path_status = None
            for st in all_statuses:
                if st.path == path:
                    path_status = st
                    break

            if path_status:
                path_status.summary = path_status.single
            else:
                path_status = rabbitvcs.vcs.status.Status.status_unknown(path)
        else:
            path_status = all_statuses[0]

        return path_status

    def is_working_copy(self, path):
        if (os.path.isdir(path) and
                os.path.isdir(os.path.join(path, ".hg"))):
            return True
        return False

    def is_in_a_or_a_working_copy(self, path):
        if self.is_working_copy(path):
            return True

        return (self.find_repository_path(os.path.split(path)[0]) != "")

    def is_versioned(self, path):
        if self.is_working_copy(path):
            return True
       
        st = self.status(path)
        try:
            return st.is_versioned()
        except Exception, e:
            log.error(e)
            return False

        return False
    
    def is_locked(self, path):
        return False

    def get_items(self, paths, statuses=[]):
        if paths is None:
            return []
        
        items = []
        for path in paths:
            st = self.statuses(path, invalidate=True)
            for st_item in st:
                if st_item.content == "modified" and os.path.isdir(st_item.path):
                    continue

                if st_item.content in statuses or len(statuses) == 0:
                    items.append(st_item)

        return items
    
    #
    # Actions
    #



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os.path

import pygtk
import gobject
import gtk

from rabbitvcs.ui import InterfaceView
from rabbitvcs.ui.checkout import SVNCheckout
from rabbitvcs.ui.clone import GitClone
from rabbitvcs.ui.dialog import MessageBox
from rabbitvcs.ui.action import SVNAction, GitAction
import rabbitvcs.util.helper

from rabbitvcs import gettext
_ = gettext.gettext

class SVNExport(SVNCheckout):
    def __init__(self, path=None, revision=None):
        SVNCheckout.__init__(self, path, url=None, revision=revision)
        
        self.svn = self.vcs.svn()
        
        self.get_widget("Checkout").set_title(_("Export - %s") % path)
        
        # Determine behavior based on the given path
        if self.svn.is_in_a_or_a_working_copy(path):
            # If path is from a working copy, export FROM path and set revision
            # to working copy
            self.repositories.set_child_text(path)
            self.get_widget("destination").set_text("")
            if revision is None:
                self.revision_selector.set_kind_working()
        elif self.svn.is_path_repository_url(path):
            # If path is a repository, export FROM path
            self.repositories.set_child_text(path)
            self.get_widget("destination").set_text("")
        else:
            # Path is not a working copy so the user probably wants to export
            # TO this path
            self.repositories.set_child_text("")
            self.get_widget("destination").set_text(path)

    def on_ok_clicked(self, widget):
        url = self.repositories.get_active_text()
        path = self._get_path()
        omit_externals = self.get_widget("omit_externals").get_active()
        recursive = self.get_widget("recursive").get_active()

        if not url or not path:
            MessageBox(_("The repository URL and destination path are both required fields."))
            return
        
        if url.startswith("file://"):
            url = self._parse_path(url)
        
        # Cannot do:
        # url = os.path.normpath(url)
        # ...in general, since it might be eg. an http URL. Doesn't seem to
        # affect pySvn though.
        
        path = os.path.normpath(path)        
        revision = self.revision_selector.get_revision_object()

        self.hide()
        self.action = SVNAction(
            self.svn,
            register_gtk_quit=self.gtk_quit_is_set()
        )
        
        self.action.append(self.action.set_header, _("Export"))
        self.action.append(self.action.set_status, _("Running Export Command..."))
        self.action.append(rabbitvcs.util.helper.save_repository_path, url)
        self.action.append(
            self.svn.export,
            url,
            path,
            force=True,
            recurse=recursive,
            revision=revision,
            ignore_externals=omit_externals
        )
        self.action.append(self.action.set_status, _("Completed Export"))
        self.action.append(self.action.finish)
        self.action.start()

class GitExport(GitClone):
    def __init__(self, path=None, revision=None):

        self.vcs = rabbitvcs.vcs.VCS()
        self.git = None
        guess = rabbitvcs.vcs.guess(path)
        if guess["vcs"] == rabbitvcs.vcs.VCS_GIT:
            self.git = self.vcs.git(path)
            export_to = ""
            export_from = path
        else:
            export_to = path
            export_from = ""

        GitClone.__init__(self, export_to, export_from)

        self.get_widget("Checkout").set_title(_("Export - %s") % path)

        self.revision_selector = rabbitvcs.ui.widget.RevisionSelector(
            self.get_widget("revision_container"),
            self.git,
            revision=revision,
            url_combobox=self.repositories,
            expand=True
        )

        self.get_widget("revision_selector_box").show()

    def on_ok_clicked(self, widget):
        url = self.repositories.get_active_text()
        path = self._get_path()

        if not url or not path:
            MessageBox(_("The repository URL and destination path are both required fields."))
            return
        
        if url.startswith("file://"):
            url = self._parse_path(url)
        
        # Cannot do:
        # url = os.path.normpath(url)
        # ...in general, since it might be eg. an http URL. Doesn't seem to
        # affect pySvn though.
        
        path = os.path.normpath(path)        
        revision = self.revision_selector.get_revision_object()

        self.hide()
        self.action = GitAction(
            self.git,
            register_gtk_quit=self.gtk_quit_is_set()
        )
        
        self.action.append(self.action.set_header, _("Export"))
        self.action.append(self.action.set_status, _("Running Export Command..."))
        self.action.append(rabbitvcs.util.helper.save_repository_path, url)
        self.action.append(
            self.git.export,
            url,
            path,
            revision=revision
        )
        self.action.append(self.action.set_status, _("Completed Export"))
        self.action.append(self.action.finish)
        self.action.start()

    def on_repositories_changed(self, widget, data=None):
        # Do not use quoting for this bit
        url = self.repositories.get_active_text()
        tmp = url.replace("//", "/").split("/")[1:]
        append = ""
        prev = ""
        while len(tmp):
            prev = append
            append = tmp.pop()
            if append not in ("trunk", "branches", "tags"):
                break
                
            if append in ("http:", "https:", "file:", "svn:", "svn+ssh:"):
                append = ""
                break
        
        self.get_widget("destination").set_text(
            os.path.join(self.destination, append)
        )
        
        self.check_form()

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNExport,
    rabbitvcs.vcs.VCS_GIT: GitExport
}

def export_factory(vcs, path, revision=None):
    if not vcs:
        guess = rabbitvcs.vcs.guess(path)
        vcs = guess["vcs"]
    
    if vcs == rabbitvcs.vcs.VCS_DUMMY:
        vcs = rabbitvcs.vcs.VCS_SVN
        
    return classes_map[vcs](path, revision)

if __name__ == "__main__":
    from rabbitvcs.ui import main, REVISION_OPT, VCS_OPT
    (options, paths) = main(
        [REVISION_OPT, VCS_OPT],
        usage="Usage: rabbitvcs export --vcs=[git|svn] [url_or_path]"
    )
            
    window = export_factory(options.vcs, paths[0], revision=options.revision)
    window.register_gtk_quit()
    gtk.main()



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os
import subprocess

import pygtk
import gobject
import gtk

import rabbitvcs.ui.dialog
from rabbitvcs.ui.action import GitAction

from rabbitvcs import gettext
_ = gettext.gettext

class SVNCreate:
    """
    Provides an interface to create a svn repository
    """
    
    # Also, might want to just launch a terminal window instead of this
    def __init__(self, path):
    
    
        if not os.path.isdir(path):
            os.makedirs(path)
        
        # Let svnadmin return a bad value if a repo already exists there
        ret = subprocess.call(["/usr/bin/svnadmin", "create", path])
        if ret == 0:
            rabbitvcs.ui.dialog.MessageBox(_("Repository successfully created"))
        else:
            rabbitvcs.ui.dialog.MessageBox(_("There was an error creating the repository.  Make sure the given folder is empty."))
        
class GitCreate:
    # Also, might want to just launch a terminal window instead of this
    def __init__(self, path):
        self.vcs = rabbitvcs.vcs.VCS()
        self.git = self.vcs.git()
        self.path = path
        
        self.action = GitAction(
            self.git,
            register_gtk_quit=True
        )
        
        self.action.append(self.action.set_header, _("Initialize Repository"))
        self.action.append(self.action.set_status, _("Setting up repository..."))
        self.action.append(self.git.initialize_repository, self.path)
        self.action.append(self.action.set_status, _("Completed repository setup"))
        self.action.append(self.action.finish)
        self.action.start()

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNCreate,
    rabbitvcs.vcs.VCS_GIT: GitCreate
}

if __name__ == "__main__":
    from rabbitvcs.ui import main, VCS_OPT, VCS_OPT_ERROR
    (options, paths) = main([VCS_OPT], usage="Usage: rabbitvcs create --vcs [svn|git] path")
    if options.vcs:
        window = classes_map[options.vcs](paths[0])
        if options.vcs == rabbitvcs.vcs.VCS_GIT:
            gtk.main()
    else:
        rabbitvcs.ui.dialog.MessageBox(VCS_OPT_ERROR)



#
# This is an extension to the Nautilus file manager to allow better
# integration with the Subversion source control system.
#
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
#
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

"""
Unit tests for the top-level rabbitvcs package.

"""

# make sure the current working copy is in sys.path before anything else
from os.path import abspath, dirname, join, normpath
import sys
toplevel = normpath(join(dirname(abspath(__file__)), '..', '..'))
sys.path.insert(0, toplevel)

from unittest import TestCase, main
import traceback

import nautilus
import pysvn
import rabbitvcs
from rabbitvcs.util.extensions.nautilus import RabbitVCS


class RabbitVCSTest(TestCase):
    """
    Main RabbitVCS tests.

    """
    def test_package_name(self):
        """Make sure the package name is reported properly."""
        result = rabbitvcs.package_name()
        self.assertEqual(result, "rabbitvcs")

    def test_package_version(self):
        """Make sure the package version is reported properly."""
        result = rabbitvcs.package_version()
        for character in result:
            if not (character.isdigit() or character == '.'):
                self.fail("Not all characters in package version "
                          "'%s' were digits or dots." % result)

    def test_package_identifier(self):
        """Make sure the package identifier is reported properly."""
        result = rabbitvcs.package_identifier()
        version = rabbitvcs.package_version()
        self.assertEqual(result, "rabbitvcs-%s" % version)


class FakeVersion(object):
    """
    Fake revision info for FakeInfo, below.

    """
    def __init__(self, number):
        self.number = number


class FakeInfo(object):
    """
    Fake pysvn.Client.info() response.

    """
    def __init__(self):
        self.data = {'text_status': pysvn.wc_status_kind.none,
                     'commit_revision': FakeVersion(1234),
                     'commit_author': None,
                     'commit_time': 0.0,
                     'url': None,
                     }


class FakeClient(object):
    """
    Fake pysvn.Client that can have its behavior controlled.

    """
    instance_count = 0
    send_empty_info = True

    def __init__(self, *args, **kwargs):
        FakeClient.instance_count += 1

    def info(self, path):
        if self.send_empty_info:
            return None
        else:
            return FakeInfo()

    def status(self, path, recurse=False):
        """Return a fake status, as a list."""
        return [FakeInfo()]


class FakeLog(object):
    """
    Fake logger that allows us to pick the log messages out from
    within unit tests.

    """
    def __init__(self, prefix):
        self.prefix = prefix
        self.messages = []

    def exception(self):
        """
        Log an exception.  Just add the (exc_type, message,
        traceback) tuple onto the list of messages.

        """
        info = sys.exc_info()
        self.messages.append(info)


class RabbitVCSPySvnTest(TestCase):
    """
    RabbitVCS tests that involve pysvn in such a way that we need to
    fiddle with pysvn stuff for the tests to work.

    """
    def setUp(self):
        self.oldClient = pysvn.Client
        pysvn.Client = FakeClient
        FakeClient.instance_count = 0
        self.oldLog = RabbitVCS.log
        self.logger = FakeLog("rabbitvcs")
        RabbitVCS.log = self.logger
        self.nsvn = RabbitVCS.RabbitVCS()

    def test_update_columns_missing_info(self):
        """
        Test the behavior of update_columns() when the info() call
        returns None.
        See http://code.google.com/p/rabbitvcs/issues/detail?id=119

        The desired behavior is that an error message is logged which
        indicates that the given path is not under source control.

        """
        path = "awesomepath"
        FakeClient.send_empty_info = True
        item = nautilus.NautilusVFSFile()
        self.nsvn.update_columns(item, path)
        self.assertEqual(FakeClient.instance_count, 2)
        self.assertEqual(len(self.logger.messages), 1)
        last_message = self.logger.messages[-1]
        self.assertEqual(str(last_message[1]),
                         "The path 'awesomepath' does not "
                         "appear to be under source control.")

    def test_update_columns_correct_info(self):
        """
        Test the side effects of update_columns() when things happen
        normally.

        """
        path = "excellentpath"
        FakeClient.send_empty_info = False
        item = nautilus.NautilusVFSFile()
        self.nsvn.update_columns(item, path)
        self.assertEqual(FakeClient.instance_count, 2)
        if len(self.logger.messages) > 0:
            for e,m,t in self.logger.messages:
                traceback.print_exception(e, m, t)
            self.fail()

    def tearDown(self):
        RabbitVCS.log = self.oldLog
        pysvn.Client = self.oldClient


if __name__ == "__main__":
    main()



#
# This is an extension to the Nautilus file manager to allow better
# integration with the Subversion source control system.
#
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
#
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os
import thread
from time import sleep

import pygtk
import gobject
import gtk

from rabbitvcs.ui import InterfaceView
from rabbitvcs.util.contextmenu import GtkFilesContextMenu, GtkContextMenuCaller
import rabbitvcs.ui.widget
import rabbitvcs.ui.dialog
import rabbitvcs.ui.action
import rabbitvcs.util.helper
import rabbitvcs.vcs
from rabbitvcs.util.log import Log
from rabbitvcs.vcs.status import Status

log = Log("rabbitvcs.ui.add")

from rabbitvcs import gettext
_ = gettext.gettext

gtk.gdk.threads_init()

class Add(InterfaceView, GtkContextMenuCaller):
    """
    Provides an interface for the user to add unversioned files to a
    repository.  Also, provides a context menu with some extra functionality.

    Send a list of paths to be added

    """

    TOGGLE_ALL = True

    def __init__(self, paths, base_dir=None):
        InterfaceView.__init__(self, "add", "Add")

        self.paths = paths
        self.base_dir = base_dir
        self.last_row_clicked = None
        self.vcs = rabbitvcs.vcs.VCS()
        self.items = []
        self.show_ignored = False

        # TODO Remove this when there is svn support
        for path in paths:
            if rabbitvcs.vcs.guess(path)['vcs'] == rabbitvcs.vcs.VCS_SVN:
                self.get_widget("show_ignored").set_sensitive(False)

        self.statuses = self.vcs.statuses_for_add(paths)
        self.files_table = rabbitvcs.ui.widget.Table(
            self.get_widget("files_table"),
            [gobject.TYPE_BOOLEAN, rabbitvcs.ui.widget.TYPE_PATH,
                gobject.TYPE_STRING],
            [rabbitvcs.ui.widget.TOGGLE_BUTTON, _("Path"), _("Extension")],
            filters=[{
                "callback": rabbitvcs.ui.widget.path_filter,
                "user_data": {
                    "base_dir": base_dir,
                    "column": 1
                }
            }],
            callbacks={
                "row-activated":  self.on_files_table_row_activated,
                "mouse-event":   self.on_files_table_mouse_event,
                "key-event":     self.on_files_table_key_event
            }
        )

        self.initialize_items()

    #
    # Helpers
    #

    def load(self):
        gtk.gdk.threads_enter()
        self.get_widget("status").set_text(_("Loading..."))
        self.items = self.vcs.get_items(self.paths, self.statuses)
        
        if self.show_ignored:
            for path in paths:
                # TODO Refactor
                # TODO SVN support
                # TODO Further fix ignore patterns
                if rabbitvcs.vcs.guess(path)['vcs'] == rabbitvcs.vcs.VCS_GIT:
                    git = self.vcs.git(path)
                    for ignored_path in git.client.get_all_ignore_file_paths(path):
                        should_add = True
                        for item in self.items:
                            if item.path == os.path.realpath(ignored_path):
                                should_add = False

                        if should_add:
                            self.items.append(Status(os.path.realpath(ignored_path), 'unversioned'))
                     


        self.populate_files_table()
        self.get_widget("status").set_text(_("Found %d item(s)") % len(self.items))
        gtk.gdk.threads_leave()

    def populate_files_table(self):
        self.files_table.clear()
        for item in self.items:
            self.files_table.append([
                True,
                item.path,
                rabbitvcs.util.helper.get_file_extension(item.path)
            ])

    def toggle_ignored(self):
        self.show_ignored = not self.show_ignored
        self.initialize_items()

    # Overrides the GtkContextMenuCaller method
    def on_context_menu_command_finished(self):
        self.initialize_items()

    def initialize_items(self):
        """
        Initializes the activated cache and loads the file items in a new thread
        """

        try:
            thread.start_new_thread(self.load, ())
        except Exception, e:
            log.exception(e)

    def delete_items(self, widget, data=None):
        paths = self.files_table.get_selected_row_items(1)
        if len(paths) > 0:
            proc = rabbitvcs.util.helper.launch_ui_window("delete", paths)
            self.rescan_after_process_exit(proc, paths)

    #
    # UI Signal Callbacks
    #

    def on_select_all_toggled(self, widget):
        self.TOGGLE_ALL = not self.TOGGLE_ALL
        for row in self.files_table.get_items():
            row[0] = self.TOGGLE_ALL

    def on_show_ignored_toggled(self, widget):
        self.toggle_ignored()                  

    def on_files_table_row_activated(self, treeview, event, col):
        paths = self.files_table.get_selected_row_items(1)
        rabbitvcs.util.helper.launch_diff_tool(*paths)

    def on_files_table_key_event(self, treeview, data=None):
        if gtk.gdk.keyval_name(data.keyval) == "Delete":
            self.delete_items(treeview, data)

    def on_files_table_mouse_event(self, treeview, data=None):
        if data is not None and data.button == 3:
            self.show_files_table_popup_menu(treeview, data)

    def show_files_table_popup_menu(self, treeview, data):
        paths = self.files_table.get_selected_row_items(1)
        GtkFilesContextMenu(self, data, self.base_dir, paths).show()


class SVNAdd(Add):
    def __init__(self, paths, base_dir=None):
        Add.__init__(self, paths, base_dir)
        
        self.svn = self.vcs.svn()

    def on_ok_clicked(self, widget):
        items = self.files_table.get_activated_rows(1)
        if not items:
            self.close()
            return

        self.hide()

        self.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            register_gtk_quit=self.gtk_quit_is_set()
        )
        self.action.append(self.action.set_header, _("Add"))
        self.action.append(self.action.set_status, _("Running Add Command..."))
        self.action.append(self.svn.add, items)
        self.action.append(self.action.set_status, _("Completed Add"))
        self.action.append(self.action.finish)
        self.action.start()

class GitAdd(Add):
    def __init__(self, paths, base_dir=None):
        Add.__init__(self, paths, base_dir)
        
        self.git = self.vcs.git(paths[0])
    
    def on_ok_clicked(self, widget):
        items = self.files_table.get_activated_rows(1)
        if not items:
            self.close()
            return

        self.hide()

        self.action = rabbitvcs.ui.action.GitAction(
            self.git,
            register_gtk_quit=self.gtk_quit_is_set()
        )
        self.action.append(self.action.set_header, _("Add"))
        self.action.append(self.action.set_status, _("Running Add Command..."))
        self.action.append(self.git.add, items)
        self.action.append(self.action.set_status, _("Completed Add"))
        self.action.append(self.action.finish)
        self.action.start()

classes_map = {
    rabbitvcs.vcs.VCS_SVN: SVNAdd,
    rabbitvcs.vcs.VCS_GIT: GitAdd
}

def add_factory(paths, base_dir):
    guess = rabbitvcs.vcs.guess (paths[0])
    return classes_map[guess["vcs"]](paths,base_dir)

class AddQuiet:
    def __init__(self, paths):
        self.vcs = rabbitvcs.vcs.VCS()
        self.svn = self.vcs.svn()
        self.action = rabbitvcs.ui.action.SVNAction(
            self.svn,
            run_in_thread=False
        )

        self.action.append(self.svn.add, paths)
        self.action.run()

if __name__ == "__main__":
    from rabbitvcs.ui import main, BASEDIR_OPT, QUIET_OPT
    (options, paths) = main(
        [BASEDIR_OPT, QUIET_OPT],
        usage="Usage: rabbitvcs add [path1] [path2] ..."
    )

    if options.quiet:
        AddQuiet(paths)
    else:
        window = add_factory(paths,options.base_dir)#Add(paths, options.base_dir)
        window.register_gtk_quit()
        gtk.main()



#
# This is an extension to the Nautilus file manager to allow better 
# integration with the Subversion source control system.
# 
# Copyright (C) 2006-2008 by Jason Field <jason@jasonfield.com>
# Copyright (C) 2007-2008 by Bruce van der Kooij <brucevdkooij@gmail.com>
# Copyright (C) 2008-2010 by Adam Plumb <adamplumb@gmail.com>
# 
# RabbitVCS is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# RabbitVCS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with RabbitVCS;  If not, see <http://www.gnu.org/licenses/>.
#

import os

import pygtk
import gobject
import gtk
import pango
import dbus

from rabbitvcs.ui import InterfaceView
import rabbitvcs.ui.widget
import rabbitvcs.ui.dialog
import rabbitvcs.util.settings
import rabbitvcs.util.helper

import rabbitvcs.services.checkerservice
from rabbitvcs.services.checkerservice import StatusCheckerStub

from rabbitvcs import gettext, _gettext, APP_NAME, LOCALE_DIR
_ = gettext.gettext

CHECKER_UNKNOWN_INFO = _("Unknown")
CHECKER_SERVICE_ERROR = _(
"There was an error communicating with the status checker service.")

from locale import getdefaultlocale

class Settings(InterfaceView):
    def __init__(self, base_dir=None):
        """
        Provides an interface to the settings library.
        """
    
        InterfaceView.__init__(self, "settings", "Settings")

        self.settings = rabbitvcs.util.settings.SettingsManager()
        
        langs = []
        language = os.environ.get('LANGUAGE', None)
        if language:
            langs += language.split(":")

        self.language = rabbitvcs.ui.widget.ComboBox(
            self.get_widget("language"), 
            langs
        )
        self.language.set_active_from_value(
            self.settings.get("general", "language")
        )
        self.get_widget("enable_attributes").set_active(
            int(self.settings.get("general", "enable_attributes"))
        )
        self.get_widget("enable_emblems").set_active(
            int(self.settings.get("general", "enable_emblems"))
        )
        self.get_widget("enable_recursive").set_active(
            int(self.settings.get("general", "enable_recursive"))
        )
        self.get_widget("show_debug").set_active(
            int(self.settings.get("general","show_debug"))
        )
        self.get_widget("diff_tool").set_text(
            self.settings.get("external", "diff_tool")
        )
        self.get_widget("diff_tool_swap").set_active(
            int(self.settings.get("external", "diff_tool_swap"))
        )
        self.get_widget("cache_number_repositories").set_text(
            str(self.settings.get("cache", "number_repositories"))
        )
        self.get_widget("cache_number_messages").set_text(
            str(self.settings.get("cache", "number_messages"))
        )
        
        self.logging_type = rabbitvcs.ui.widget.ComboBox(
            self.get_widget("logging_type"), 
            ["None", "Console", "File", "Both"]
        )
        val = self.settings.get("logging", "type")
        if not val:
            val = "Console"
        self.logging_type.set_active_from_value(val)

        self.logging_level = rabbitvcs.ui.widget.ComboBox(
            self.get_widget("logging_level"), 
            ["Debug", "Info", "Warning", "Error", "Critical"]
        )
        val = self.settings.get("logging", "level")
        if not val:
            val = "Debug"
        self.logging_level.set_active_from_value(val)
        
        # Git Configuration Editor
        show_git = False
        self.file_editor = None
        if base_dir:
            vcs = rabbitvcs.vcs.VCS()
            git_config_files = []
            if vcs.is_in_a_or_a_working_copy(base_dir) and vcs.guess(base_dir)["vcs"] == rabbitvcs.vcs.VCS_GIT:
                git = vcs.git(base_dir)
                git_config_files = git.get_config_files(base_dir)
        
                self.file_editor = rabbitvcs.ui.widget.MultiFileTextEditor(
                    self.get_widget("git_config_container"),
                    _("Config file:"),
                    git_config_files,
                    git_config_files,
                    show_add_line=False
                )
                show_git = True

        if show_git:
            self.get_widget("pages").get_nth_page(5).show()
        else:
            self.get_widget("pages").get_nth_page(5).hide()

        self._populate_checker_tab()

    def _get_checker_service(self, report_failure=True):
        checker_service = None
        try:
            session_bus = dbus.SessionBus()
            checker_service = session_bus.get_object(
                                    rabbitvcs.services.checkerservice.SERVICE,
                                    rabbitvcs.services.checkerservice.OBJECT_PATH)
        except dbus.DBusException, ex:
            if report_failure:
                rabbitvcs.ui.dialog.MessageBox(CHECKER_SERVICE_ERROR)
        
        return checker_service

    def _populate_checker_tab(self, report_failure=True):
        # This is a limitation of GLADE, and can be removed when we migrate to
        # GTK2 Builder

        checker_service = self._get_checker_service(report_failure)
        
        self.get_widget("restart_checker").set_image(
                                        gtk.image_new_from_stock(
                                            gtk.STOCK_EXECUTE,
                                            gtk.ICON_SIZE_BUTTON))

        self.get_widget("refresh_info").set_image(
                                        gtk.image_new_from_stock(
                                            gtk.STOCK_REFRESH,
                                            gtk.ICON_SIZE_BUTTON))

        self.get_widget("stop_checker").set_image(
                                        gtk.image_new_from_stock(
                                            gtk.STOCK_STOP,
                                            gtk.ICON_SIZE_BUTTON))

        self.get_widget("stop_checker").set_sensitive(bool(checker_service))

        if(checker_service):
            self.get_widget("checker_type").set_text(checker_service.CheckerType())
            self.get_widget("pid").set_text(str(checker_service.PID()))
            
            memory = checker_service.MemoryUsage()
                        
            if memory:
                self.get_widget("memory_usage").set_text("%s KB" % memory)
            else:
                self.get_widget("memory_usage").set_text(CHECKER_UNKNOWN_INFO)
            
            self._populate_info_table(checker_service.ExtraInformation())
            
        else:
            self.get_widget("checker_type").set_text(CHECKER_UNKNOWN_INFO)
            self.get_widget("pid").set_text(CHECKER_UNKNOWN_INFO)
            self.get_widget("memory_usage").set_text(CHECKER_UNKNOWN_INFO)
            self._clear_info_table()

    def _clear_info_table(self):
        info_table = self.get_widget("info_table_area").get_child()
        
        if info_table:
            info_table.destroy()

    def _populate_info_table(self, info):
        self._clear_info_table()
        
        table_place = self.get_widget("info_table_area")
        
        table = rabbitvcs.ui.widget.KeyValueTable(info)
        table_place.add(table)
        table.show()
        
    
    def on_refresh_info_clicked(self, widget):
        self._populate_checker_tab()
    
    def _stop_checker(self):
        checker_service = self._get_checker_service(False)
        pid = None
        if(checker_service):
            try:
                pid = checker_service.Quit()
            except dbus.exceptions.DBusException:
                # Ignore it, it will necessarily happen when we kill the service
                pass
        if pid:
            try:
                os.waitpid(pid, 0)
            except OSError:
                # This occurs if the process is already gone.
                pass
    
    def on_restart_checker_clicked(self, widget):
        self._stop_checker()
        rabbitvcs.services.checkerservice.start()
        self._populate_checker_tab()
    
    def on_stop_checker_clicked(self, widget):
        self._stop_checker()
        self._populate_checker_tab(report_failure=False)

    def on_destroy(self, widget):
        gtk.main_quit()

    def on_cancel_clicked(self, widget):
        gtk.main_quit()

    def on_ok_clicked(self, widget):
        self.save()
        gtk.main_quit()
    
    def on_apply_clicked(self, widget):
        self.save()

    def save(self):
        self.settings.set(
            "general", "language", 
            self.get_widget("language").get_active_text()
        )
        self.settings.set(
            "general", "enable_attributes",
            self.get_widget("enable_attributes").get_active()
        )
        self.settings.set(
            "general", "enable_emblems",
            self.get_widget("enable_emblems").get_active()
        )
        self.settings.set(
            "general", "enable_recursive",
            self.get_widget("enable_recursive").get_active()
        )
        self.settings.set(
            "general", "show_debug",
            self.get_widget("show_debug").get_active()
        )
        self.settings.set(
            "external", "diff_tool",
            self.get_widget("diff_tool").get_text()
        )
        self.settings.set(
            "external", "diff_tool_swap",
            self.get_widget("diff_tool_swap").get_active()
        )
        self.settings.set(
            "cache", "number_repositories",
            self.get_widget("cache_number_repositories").get_text()
        )
        self.settings.set(
            "cache", "number_messages",
            self.get_widget("cache_number_messages").get_text()
        )
        self.settings.set(
            "logging", "type",
            self.logging_type.get_active_text()
        )
        self.settings.set(
            "logging", "level",
            self.logging_level.get_active_text()
        )
        self.settings.write()
        
        if self.file_editor:
            self.file_editor.save()

    def on_external_diff_tool_browse_clicked(self, widget):
        chooser = rabbitvcs.ui.dialog.FileChooser(
            _("Select a program"), "/usr/bin"
        )
        path = chooser.run()
        path = path.replace("file://", "")
        if path is not None:
            self.get_widget("diff_tool").set_text(path)

    def on_cache_clear_repositories_clicked(self, widget):
        confirmation = rabbitvcs.ui.dialog.Confirmation(
            _("Are you sure you want to clear your repository paths?")
        )
        if confirmation.run() == gtk.RESPONSE_OK:
            path = rabbitvcs.util.helper.get_repository_paths_path()
            fh = open(path, "w")
            fh.write("")
            fh.close()
            rabbitvcs.ui.dialog.MessageBox(_("Repository paths cleared"))

    def on_cache_clear_messages_clicked(self, widget):
        confirmation = rabbitvcs.ui.dialog.Confirmation(
            _("Are you sure you want to clear your previous messages?")
        )
        if confirmation.run() == gtk.RESPONSE_OK:
            path = rabbitvcs.util.helper.get_previous_messages_path()
            fh = open(path, "w")
            fh.write("")
            fh.close()
            rabbitvcs.ui.dialog.MessageBox(_("Previous messages cleared"))

    def on_cache_clear_authentication_clicked(self, widget):
        confirmation = rabbitvcs.ui.dialog.Confirmation(
            _("Are you sure you want to clear your authentication information?")
        )
        if confirmation.run() == gtk.RESPONSE_OK:
            home_dir = rabbitvcs.util.helper.get_user_path()
            subpaths = [
                '/.subversion/auth/svn.simple',
                '/.subversion/auth/svn.ssl.server',
                '/.subversion/auth/svn.username'
            ]
            for subpath in subpaths:
                path = "%s%s" % (home_dir, subpath)
                if os.path.exists(path):
                    files = os.listdir(path)
                    for filename in files:
                        filepath = "%s/%s" % (path, filename)
                        os.remove(filepath)

            rabbitvcs.ui.dialog.MessageBox(_("Authentication information cleared"))
            

if __name__ == "__main__":
    from rabbitvcs.ui import main, BASEDIR_OPT
    (options, paths) = main(
        [BASEDIR_OPT],
        usage="Usage: rabbitvcs settings"
    )

    window = Settings(options.base_dir)
    window.register_gtk_quit()
    gtk.main()
